"""
This module defines the ASE interface to SIESTA.

Written by Mads Engelund (see www.espeem.com)

Home of the SIESTA package:
http://www.uam.es/departamentos/ciencias/fismateriac/siesta

2017.04 - Pedro Brandimarte: changes for python 2-3 compatible

"""

import os
import re
import tempfile
import warnings
import shutil
from os.path import join, isfile, islink
import numpy as np

from ase.units import Ry, eV, Bohr
from ase.data import atomic_numbers
from ase.calculators.siesta.import_functions import read_rho, xv_to_atoms
from ase.calculators.siesta.import_functions import \
    get_valence_charge, read_vca_synth_block
from ase.calculators.calculator import FileIOCalculator, ReadError
from ase.calculators.calculator import Parameters, all_changes
from ase.calculators.siesta.parameters import PAOBasisBlock, Species
from ase.calculators.siesta.parameters import format_fdf


meV = 0.001 * eV


def get_siesta_version(command):
    """ Return SIESTA version number.

    Run the command, for instance 'siesta' and
    then parse the output in order find the
    version number.
    """

    temp_dirname = tempfile.mkdtemp(prefix='siesta-version-check-')
    try:
        from subprocess import Popen, PIPE
        proc = Popen(command,
                     stdin=PIPE,
                     stdout=PIPE,
                     stderr=PIPE,
                     cwd=temp_dirname)
        output, _ = proc.communicate()
        # We are not providing any input, so Siesta will give us a failure
        # saying that it has no Chemical_species_label and exit status 1
        # (as of siesta-4.1-b4)
    finally:
        shutil.rmtree(temp_dirname)

    match = re.search(rb'Siesta Version\s*:\s*(\S+)', output)

    if match is None:
        raise RuntimeError('Could not get Siesta version info from output '
                           '{!r}'.format(output))

    string = match.group(1).decode('ascii')
    return string


def bandpath2bandpoints(path):
    lines = []
    add = lines.append

    add('BandLinesScale ReciprocalLatticeVectors\n')
    add('%block BandPoints\n')
    for kpt in path.kpts:
        add('    {:18.15f} {:18.15f} {:18.15f}\n'.format(*kpt))
    add('%endblock BandPoints')
    return ''.join(lines)


def read_bands_file(fd):
    efermi = float(next(fd))
    next(fd)  # Appears to be max/min energy.  Not important for us
    header = next(fd)  # Array shape: nbands, nspins, nkpoints
    nbands, nspins, nkpts = np.array(header.split()).astype(int)

    # three fields for kpt coords, then all the energies
    ntokens = nbands * nspins + 3

    # Read energies for each kpoint:
    data = []
    for i in range(nkpts):
        line = next(fd)
        tokens = line.split()
        while len(tokens) < ntokens:
            # Multirow table.  Keep adding lines until the table ends,
            # which should happen exactly when we have all the energies
            # for this kpoint.
            line = next(fd)
            tokens += line.split()
        assert len(tokens) == ntokens
        values = np.array(tokens).astype(float)
        data.append(values)

    data = np.array(data)
    assert len(data) == nkpts
    kpts = data[:, :3]
    energies = data[:, 3:]
    energies = energies.reshape(nkpts, nspins, nbands)
    assert energies.shape == (nkpts, nspins, nbands)
    return kpts, energies, efermi


def resolve_band_structure(path, kpts, energies, efermi):
    """Convert input BandPath along with Siesta outputs into BS object."""
    # Right now this function doesn't do much.
    #
    # Not sure how the output kpoints in the siesta.bands file are derived.
    # They appear to be related to the lattice parameter.
    #
    # We should verify that they are consistent with our input path,
    # but since their meaning is unclear, we can't quite do so.
    #
    # Also we should perhaps verify the cell.  If we had the cell, we
    # could construct the bandpath from scratch (i.e., pure outputs).
    from ase.dft.band_structure import BandStructure
    ksn2e = energies
    skn2e = np.swapaxes(ksn2e, 0, 1)
    bs = BandStructure(path, skn2e, reference=efermi)
    return bs


class SiestaParameters(Parameters):
    """Parameters class for the calculator.
    Documented in BaseSiesta.__init__

    """
    def __init__(
            self,
            label='siesta',
            mesh_cutoff=200 * Ry,
            energy_shift=100 * meV,
            kpts=None,
            xc='LDA',
            basis_set='DZP',
            spin='non-polarized',
            species=tuple(),
            pseudo_qualifier=None,
            pseudo_path=None,
            symlink_pseudos=None,
            atoms=None,
            restart=None,
            ignore_bad_restart_file=False,
            fdf_arguments=None,
            atomic_coord_format='xyz',
            bandpath=None):
        kwargs = locals()
        kwargs.pop('self')
        Parameters.__init__(self, **kwargs)


class Siesta(FileIOCalculator):
    """Calculator interface to the SIESTA code.
    """
    allowed_basis_names = ['SZ', 'SZP', 'DZ', 'DZP', 'TZP']
    allowed_spins = ['non-polarized', 'collinear',
                     'non-collinear', 'spin-orbit']
    allowed_xc = {
        'LDA': ['PZ', 'CA', 'PW92'],
        'GGA': ['PW91', 'PBE', 'revPBE', 'RPBE',
                'WC', 'AM05', 'PBEsol', 'PBEJsJrLO',
                'PBEGcGxLO', 'PBEGcGxHEG', 'BLYP'],
        'VDW': ['DRSLL', 'LMKLL', 'KBM', 'C09', 'BH', 'VV']}

    name = 'siesta'
    command = 'siesta < PREFIX.fdf > PREFIX.out'
    implemented_properties = (
        'energy',
        'forces',
        'stress',
        'dipole',
        'eigenvalues',
        'density',
        'fermi_energy')

    # Dictionary of valid input vaiables.
    default_parameters = SiestaParameters()

    # XXX Not a ASE standard mechanism (yet).  We need to communicate to
    # ase.dft.band_structure.calculate_band_structure() that we expect
    # it to use the bandpath keyword.
    accepts_bandpath_keyword = True

    def __init__(self, command=None, **kwargs):
        """ASE interface to the SIESTA code.

        Parameters:
           - label        : The basename of all files created during
                            calculation.
           - mesh_cutoff  : Energy in eV.
                            The mesh cutoff energy for determining number of
                            grid points in the matrix-element calculation.
           - energy_shift : Energy in eV
                            The confining energy of the basis set generation.
           - kpts         : Tuple of 3 integers, the k-points in different
                            directions.
           - xc           : The exchange-correlation potential. Can be set to
                            any allowed value for either the Siesta
                            XC.funtional or XC.authors keyword. Default "LDA"
           - basis_set    : "SZ"|"SZP"|"DZ"|"DZP"|"TZP", strings which specify
                            the type of functions basis set.
           - spin         : "non-polarized"|"collinear"|
                            "non-collinear|spin-orbit".
                            The level of spin description to be used.
           - species      : None|list of Species objects. The species objects
                            can be used to to specify the basis set,
                            pseudopotential and whether the species is ghost.
                            The tag on the atoms object and the element is used
                            together to identify the species.
           - pseudo_path  : None|path. This path is where
                            pseudopotentials are taken from.
                            If None is given, then then the path given
                            in $SIESTA_PP_PATH will be used.
           - pseudo_qualifier: None|string. This string will be added to the
                            pseudopotential path that will be retrieved.
                            For hydrogen with qualifier "abc" the
                            pseudopotential "H.abc.psf" will be retrieved.
           - symlink_pseudos: None|bool
                            If true, symlink pseudopotentials
                            into the calculation directory, else copy them.
                            Defaults to true on Unix and false on Windows.
           - atoms        : The Atoms object.
           - restart      : str.  Prefix for restart file.
                            May contain a directory.
                            Default is  None, don't restart.
           - ignore_bad_restart_file: bool.
                            Ignore broken or missing restart file.
                            By default, it is an error if the restart
                            file is missing or broken.
           - fdf_arguments: Explicitly given fdf arguments. Dictonary using
                            Siesta keywords as given in the manual. List values
                            are written as fdf blocks with each element on a
                            separate line, while tuples will write each element
                            in a single line.  ASE units are assumed in the
                            input.
           - atomic_coord_format: "xyz"|"zmatrix", strings to switch between
                            the default way of entering the system's geometry
                            (via the block AtomicCoordinatesAndAtomicSpecies)
                            and a recent method via the block Zmatrix. The
                            block Zmatrix allows to specify basic geometry
                            constrains such as realized through the ASE classes
                            FixAtom, FixedLine and FixedPlane.
        """

        # Put in the default arguments.
        parameters = self.default_parameters.__class__(**kwargs)

        # Call the base class.
        FileIOCalculator.__init__(
            self,
            command=command,
            **parameters)

        # For compatibility with old variable name:
        commandvar = os.environ.get('SIESTA_COMMAND')
        if commandvar is not None:
            warnings.warn('Please use $ASE_SIESTA_COMMAND and not '
                          '$SIESTA_COMMAND, which will be ignored '
                          'in the future.  The new command format will not '
                          'work with the "<%s > %s" specification.  Use '
                          'instead e.g. "ASE_SIESTA_COMMAND=siesta'
                          ' < PREFIX.fdf > PREFIX.out", where PREFIX will '
                          'automatically be replaced by calculator label',
                          np.VisibleDeprecationWarning)
            runfile = self.prefix + '.fdf'
            outfile = self.prefix + '.out'
            try:
                self.command = commandvar % (runfile, outfile)
            except TypeError:
                raise ValueError(
                    "The 'SIESTA_COMMAND' environment must " +
                    "be a format string" +
                    " with two string arguments.\n" +
                    "Example : 'siesta < %s > %s'.\n" +
                    "Got '%s'" % commandvar)

    def __getitem__(self, key):
        """Convenience method to retrieve a parameter as
        calculator[key] rather than calculator.parameters[key]

            Parameters:
                -key       : str, the name of the parameters to get.
        """
        return self.parameters[key]

    def species(self, atoms):
        """Find all relevant species depending on the atoms object and
        species input.

            Parameters :
                - atoms : An Atoms object.
        """
        # For each element use default species from the species input, or set
        # up a default species  from the general default parameters.
        symbols = np.array(atoms.get_chemical_symbols())
        tags = atoms.get_tags()
        species = list(self['species'])
        default_species = [
            s for s in species
            if (s['tag'] is None) and s['symbol'] in symbols]
        default_symbols = [s['symbol'] for s in default_species]
        for symbol in symbols:
            if symbol not in default_symbols:
                spec = Species(symbol=symbol,
                               basis_set=self['basis_set'],
                               tag=None)
                default_species.append(spec)
                default_symbols.append(symbol)
        assert len(default_species) == len(np.unique(symbols))

        # Set default species as the first species.
        species_numbers = np.zeros(len(atoms), int)
        i = 1
        for spec in default_species:
            mask = symbols == spec['symbol']
            species_numbers[mask] = i
            i += 1

        # Set up the non-default species.
        non_default_species = [s for s in species if not s['tag'] is None]
        for spec in non_default_species:
            mask1 = (tags == spec['tag'])
            mask2 = (symbols == spec['symbol'])
            mask = np.logical_and(mask1, mask2)
            if sum(mask) > 0:
                species_numbers[mask] = i
                i += 1
        all_species = default_species + non_default_species

        return all_species, species_numbers

    def set(self, **kwargs):
        """Set all parameters.

            Parameters:
                -kwargs  : Dictionary containing the keywords defined in
                           SiestaParameters.
        """

        # Find not allowed keys.
        default_keys = list(self.__class__.default_parameters)
        offending_keys = set(kwargs) - set(default_keys)
        if len(offending_keys) > 0:
            mess = "'set' does not take the keywords: %s "
            raise ValueError(mess % list(offending_keys))

        # Use the default parameters.
        parameters = self.__class__.default_parameters.copy()
        parameters.update(kwargs)
        kwargs = parameters

        # Check energy inputs.
        for arg in ['mesh_cutoff', 'energy_shift']:
            value = kwargs.get(arg)
            if value is None:
                continue
            if not (isinstance(value, (float, int)) and value > 0):
                mess = "'%s' must be a positive number(in eV), \
                    got '%s'" % (arg, value)
                raise ValueError(mess)

        # Check the basis set input.
        if 'basis_set' in kwargs:
            basis_set = kwargs['basis_set']
            allowed = self.allowed_basis_names
            if not (isinstance(basis_set, PAOBasisBlock) or
                    basis_set in allowed):
                mess = "Basis must be either %s, got %s" % (allowed, basis_set)
                raise ValueError(mess)

        # Check the spin input.
        if 'spin' in kwargs:
            if kwargs['spin'] == 'UNPOLARIZED':
                warnings.warn("The keyword 'UNPOLARIZED' is deprecated,"
                              "and replaced by 'non-polarized'",
                              np.VisibleDeprecationWarning)
                kwargs['spin'] = 'non-polarized'

            spin = kwargs['spin']
            if spin is not None and (spin.lower() not in self.allowed_spins):
                mess = "Spin must be %s, got '%s'" % (self.allowed_spins, spin)
                raise ValueError(mess)

        # Check the functional input.
        xc = kwargs.get('xc', 'LDA')
        if isinstance(xc, (tuple, list)) and len(xc) == 2:
            functional, authors = xc
            if functional.lower() not in [k.lower() for k in self.allowed_xc]:
                mess = "Unrecognized functional keyword: '%s'" % functional
                raise ValueError(mess)

            lsauthorslower = [a.lower() for a in self.allowed_xc[functional]]
            if authors.lower() not in lsauthorslower:
                mess = "Unrecognized authors keyword for %s: '%s'"
                raise ValueError(mess % (functional, authors))

        elif xc in self.allowed_xc:
            functional = xc
            authors = self.allowed_xc[xc][0]
        else:
            found = False
            for key, value in self.allowed_xc.items():
                if xc in value:
                    found = True
                    functional = key
                    authors = xc
                    break

            if not found:
                raise ValueError("Unrecognized 'xc' keyword: '%s'" % xc)
        kwargs['xc'] = (functional, authors)

        # Check fdf_arguments.
        if kwargs['fdf_arguments'] is None:
            kwargs['fdf_arguments'] = {}

        if not isinstance(kwargs['fdf_arguments'], dict):
            raise TypeError("fdf_arguments must be a dictionary.")

        # Call baseclass.
        FileIOCalculator.set(self, **kwargs)

    def set_fdf_arguments(self, fdf_arguments):
        """ Set the fdf_arguments after the initialization of the
            calculator.
        """
        self.validate_fdf_arguments(fdf_arguments)
        FileIOCalculator.set(self, fdf_arguments=fdf_arguments)

    def validate_fdf_arguments(self, fdf_arguments):
        """ Raises error if the fdf_argument input is not a
            dictionary of allowed keys.
        """
        # None is valid
        if fdf_arguments is None:
            return

        # Type checking.
        if not isinstance(fdf_arguments, dict):
            raise TypeError("fdf_arguments must be a dictionary.")

    def calculate(self,
                  atoms=None,
                  properties=['energy'],
                  system_changes=all_changes):
        """Capture the RuntimeError from FileIOCalculator.calculate
        and add a little debug information from the Siesta output.

        See base FileIocalculator for documentation.
        """

        FileIOCalculator.calculate(
            self,
            atoms=atoms,
            properties=properties,
            system_changes=system_changes)

        # The below snippet would run if calculate() failed but I have
        # disabled it for now since it looks to be just for debugging.
        # --askhl
        """
        # Here a test to check if the potential are in the right place!!!
        except RuntimeError as e:
            try:
                fname = os.path.join(self.directory, self.label+'.out')
                with open(fname, 'r') as f:
                    lines = f.readlines()
                debug_lines = 10
                print('##### %d last lines of the Siesta output' % debug_lines)
                for line in lines[-20:]:
                    print(line.strip())
                print('##### end of siesta output')
                raise e
            except:
                raise e
        """

    def write_input(self, atoms, properties=None, system_changes=None):
        """Write input (fdf)-file.
        See calculator.py for further details.

        Parameters:
            - atoms        : The Atoms object to write.
            - properties   : The properties which should be calculated.
            - system_changes : List of properties changed since last run.
        """
        # Call base calculator.
        FileIOCalculator.write_input(
            self,
            atoms=atoms,
            properties=properties,
            system_changes=system_changes)

        if system_changes is None and properties is None:
            return

        filename = self.getpath(ext='fdf')

        # On any changes, remove all analysis files.
        if system_changes is not None:
            self.remove_analysis()

        # Start writing the file.
        with open(filename, 'w') as f:
            # Write system name and label.
            f.write(format_fdf('SystemName', self.prefix))
            f.write(format_fdf('SystemLabel', self.prefix))
            f.write("\n")

            # Write explicitly given options first to
            # allow the user to override anything.
            fdf_arguments = self['fdf_arguments']
            keys = sorted(fdf_arguments.keys())
            for key in keys:
                f.write(format_fdf(key, fdf_arguments[key]))

            # Force siesta to return error on no convergence.
            # as default consistent with ASE expectations.
            if 'SCFMustConverge' not in fdf_arguments.keys():
                f.write(format_fdf('SCFMustConverge', True))
            f.write("\n")

            # Write spin level.
            f.write(format_fdf('Spin     ', self['spin']))
            # Spin backwards compatibility.
            if self['spin'] == 'collinear':
                f.write(format_fdf('SpinPolarized', (True, "# Backwards compatibility.")))
            elif self['spin'] == 'non-collinear':
                f.write(format_fdf('NonCollinear', (True, "# Backwards compatibility.")))



            # Write functional.
            functional, authors = self['xc']
            f.write(format_fdf('XC.functional', functional))
            f.write(format_fdf('XC.authors', authors))
            f.write("\n")

            # Write mesh cutoff and energy shift.
            f.write(format_fdf('MeshCutoff',
                               (self['mesh_cutoff'], 'eV')))
            f.write(format_fdf('PAO.EnergyShift',
                               (self['energy_shift'], 'eV')))
            f.write("\n")

            # Write the minimal arg
            self._write_species(f, atoms)
            self._write_structure(f, atoms)

            # Use the saved density matrix if only 'cell' and 'positions'
            # have changed.
            if (system_changes is None or
                ('numbers' not in system_changes and
                 'initial_magmoms' not in system_changes and
                 'initial_charges' not in system_changes)):
                f.write(format_fdf('DM.UseSaveDM', True))

            # Save density.
            if 'density' in properties:
                f.write(format_fdf('SaveRho', True))

            self._write_kpts(f)

            if self['bandpath'] is not None:
                lines = bandpath2bandpoints(self['bandpath'])
                f.write(lines)
                f.write('\n')

    def read(self, filename):
        """Read structural parameters from file .XV file
           Read other results from other files
           filename : siesta.XV
        """

        fname = self.getpath(filename)
        if not os.path.exists(fname):
            raise ReadError("The restart file '%s' does not exist" % fname)
        self.atoms = xv_to_atoms(fname)
        self.read_results()

    def getpath(self, fname=None, ext=None):
        """ Returns the directory/fname string """
        if fname is None:
            fname = self.prefix
        if ext is not None:
            fname = '{}.{}'.format(fname, ext)
        return os.path.join(self.directory, fname)

    def remove_analysis(self):
        """ Remove all analysis files"""
        filename = self.getpath(ext='RHO')
        if os.path.exists(filename):
            os.remove(filename)

    def _write_structure(self, f, atoms):
        """Translate the Atoms object to fdf-format.

        Parameters:
            - f:     An open file object.
            - atoms: An atoms object.
        """
        cell = atoms.cell
        f.write('\n')

        if cell.rank in [1, 2]:
            raise ValueError('Expected 3D unit cell or no unit cell.  You may '
                             'wish to add vacuum along some directions.')

        # Write lattice vectors
        if np.any(cell):
            f.write(format_fdf('LatticeConstant', '1.0 Ang'))
            f.write('%block LatticeVectors\n')
            for i in range(3):
                for j in range(3):
                    s = ('    %.15f' % cell[i, j]).rjust(16) + ' '
                    f.write(s)
                f.write('\n')
            f.write('%endblock LatticeVectors\n')
            f.write('\n')

        self._write_atomic_coordinates(f, atoms)

        # Write magnetic moments.
        magmoms = atoms.get_initial_magnetic_moments()

        # The DM.InitSpin block must be written to initialize to
        # no spin. SIESTA default is FM initialization, if the
        # block is not written, but  we must conform to the
        # atoms object.
        if self['spin'] != 'non-polarized':
            if sum(abs(magmoms)) == 0:
                f.write('#Empty block forces ASE initialization.\n')

            f.write('%block DM.InitSpin\n')
            for n, M in enumerate(magmoms):
                if M != 0:
                    f.write('    %d %.14f\n' % (n + 1, M))
            f.write('%endblock DM.InitSpin\n')
            f.write('\n')

    def _write_atomic_coordinates(self, f, atoms):
        """Write atomic coordinates.

        Parameters:
            - f:     An open file object.
            - atoms: An atoms object.
        """
        af = self.parameters.atomic_coord_format.lower()
        if af == 'xyz':
            self._write_atomic_coordinates_xyz(f, atoms)
        elif af == 'zmatrix':
            self._write_atomic_coordinates_zmatrix(f, atoms)
        else:
            raise RuntimeError('Unknown atomic_coord_format: {}'.format(af))

    def _write_atomic_coordinates_xyz(self, f, atoms):
        """Write atomic coordinates.

        Parameters:
            - f:     An open file object.
            - atoms: An atoms object.
        """
        species, species_numbers = self.species(atoms)
        f.write('\n')
        f.write('AtomicCoordinatesFormat  Ang\n')
        f.write('%block AtomicCoordinatesAndAtomicSpecies\n')
        for atom, number in zip(atoms, species_numbers):
            xyz = atom.position
            line = ('    %.9f' % xyz[0]).rjust(16) + ' '
            line += ('    %.9f' % xyz[1]).rjust(16) + ' '
            line += ('    %.9f' % xyz[2]).rjust(16) + ' '
            line += str(number) + '\n'
            f.write(line)
        f.write('%endblock AtomicCoordinatesAndAtomicSpecies\n')
        f.write('\n')

        origin = tuple(-atoms.get_celldisp().flatten())
        if any(origin):
            f.write('%block AtomicCoordinatesOrigin\n')
            f.write('     %.4f  %.4f  %.4f\n' % origin)
            f.write('%endblock AtomicCoordinatesOrigin\n')
            f.write('\n')

    def _write_atomic_coordinates_zmatrix(self, f, atoms):
        """Write atomic coordinates in Z-matrix format.

        Parameters:
            - f:     An open file object.
            - atoms: An atoms object.
        """
        species, species_numbers = self.species(atoms)
        f.write('\n')
        f.write('ZM.UnitsLength   Ang\n')
        f.write('%block Zmatrix\n')
        f.write('  cartesian\n')
        fstr = "{:5d}" + "{:20.10f}" * 3 + "{:3d}" * 3 + "{:7d} {:s}\n"
        a2constr = self.make_xyz_constraints(atoms)
        a2p, a2s = atoms.get_positions(), atoms.get_chemical_symbols()
        for ia, (sp, xyz, ccc, sym) in enumerate(zip(species_numbers,
                                                     a2p,
                                                     a2constr,
                                                     a2s)):
            f.write(fstr.format(
                sp, xyz[0], xyz[1], xyz[2], ccc[0],
                ccc[1], ccc[2], ia + 1, sym))
        f.write('%endblock Zmatrix\n')

        origin = tuple(-atoms.get_celldisp().flatten())
        if any(origin):
            f.write('%block AtomicCoordinatesOrigin\n')
            f.write('     %.4f  %.4f  %.4f\n' % origin)
            f.write('%endblock AtomicCoordinatesOrigin\n')
            f.write('\n')

    def make_xyz_constraints(self, atoms):
        """ Create coordinate-resolved list of constraints [natoms, 0:3]
        The elements of the list must be integers 0 or 1
          1 -- means that the coordinate will be updated during relaxation
          0 -- mains that the coordinate will be fixed during relaxation
        """
        from ase.constraints import FixAtoms, FixedLine, FixedPlane
        import sys
        import warnings

        a = atoms
        a2c = np.ones((len(a), 3), dtype=int)
        for c in a.constraints:
            if isinstance(c, FixAtoms):
                a2c[c.get_indices()] = 0
            elif isinstance(c, FixedLine):
                norm_dir = c.dir / np.linalg.norm(c.dir)
                if (max(norm_dir) - 1.0) > 1e-6:
                    raise RuntimeError(
                        'norm_dir: {} -- must be one of the Cartesian axes...'
                        .format(norm_dir))
                a2c[c.a] = norm_dir.round().astype(int)
            elif isinstance(c, FixedPlane):
                norm_dir = c.dir / np.linalg.norm(c.dir)
                if (max(norm_dir) - 1.0) > 1e-6:
                    raise RuntimeError(
                        'norm_dir: {} -- must be one of the Cartesian axes...'
                        .format(norm_dir))
                a2c[c.a] = abs(1 - norm_dir.round().astype(int))
            else:
                warnings.warn('Constraint {} is ignored at {}'
                              .format(str(c), sys._getframe().f_code))
        return a2c

    def _write_kpts(self, f):
        """Write kpts.

        Parameters:
            - f : Open filename.
        """
        if self["kpts"] is None:
            return
        kpts = np.array(self['kpts'])
        f.write('\n')
        f.write('#KPoint grid\n')
        f.write('%block kgrid_Monkhorst_Pack\n')

        for i in range(3):
            s = ''
            if i < len(kpts):
                number = kpts[i]
                displace = 0.0
            else:
                number = 1
                displace = 0
            for j in range(3):
                if j == i:
                    write_this = number
                else:
                    write_this = 0
                s += '     %d  ' % write_this
            s += '%1.1f\n' % displace
            f.write(s)
        f.write('%endblock kgrid_Monkhorst_Pack\n')
        f.write('\n')

    def _write_species(self, f, atoms):
        """Write input related the different species.

        Parameters:
            - f:     An open file object.
            - atoms: An atoms object.
        """
        species, species_numbers = self.species(atoms)

        if not self['pseudo_path'] is None:
            pseudo_path = self['pseudo_path']
        elif 'SIESTA_PP_PATH' in os.environ:
            pseudo_path = os.environ['SIESTA_PP_PATH']
        else:
            mess = "Please set the environment variable 'SIESTA_PP_PATH'"
            raise Exception(mess)

        f.write(format_fdf('NumberOfSpecies', len(species)))
        f.write(format_fdf('NumberOfAtoms', len(atoms)))

        pao_basis = []
        chemical_labels = []
        basis_sizes = []
        synth_blocks = []
        for species_number, spec in enumerate(species):
            species_number += 1
            symbol = spec['symbol']
            atomic_number = atomic_numbers[symbol]

            if spec['pseudopotential'] is None:
                if self.pseudo_qualifier() == '':
                    label = symbol
                    pseudopotential = label + '.psf'
                else:
                    label = '.'.join([symbol, self.pseudo_qualifier()])
                    pseudopotential = label + '.psf'
            else:
                pseudopotential = spec['pseudopotential']
                label = os.path.basename(pseudopotential)
                label = '.'.join(label.split('.')[:-1])

            if not os.path.isabs(pseudopotential):
                pseudopotential = join(pseudo_path, pseudopotential)

            if not os.path.exists(pseudopotential):
                mess = "Pseudopotential '%s' not found" % pseudopotential
                raise RuntimeError(mess)

            name = os.path.basename(pseudopotential)
            name = name.split('.')
            name.insert(-1, str(species_number))
            if spec['ghost']:
                name.insert(-1, 'ghost')
                atomic_number = -atomic_number

            name = '.'.join(name)
            pseudo_targetpath = self.getpath(name)

            if join(os.getcwd(), name) != pseudopotential:
                if islink(pseudo_targetpath) or isfile(pseudo_targetpath):
                    os.remove(pseudo_targetpath)
                symlink_pseudos = self['symlink_pseudos']

                if symlink_pseudos is None:
                    symlink_pseudos = not os.name == 'nt'

                if symlink_pseudos:
                    os.symlink(pseudopotential, pseudo_targetpath)
                else:
                    shutil.copy(pseudopotential, pseudo_targetpath)

            if not spec['excess_charge'] is None:
                atomic_number += 200
                n_atoms = sum(np.array(species_numbers) == species_number)

                paec = float(spec['excess_charge']) / n_atoms
                vc = get_valence_charge(pseudopotential)
                fraction = float(vc + paec) / vc
                pseudo_head = name[:-4]
                fractional_command = os.environ['SIESTA_UTIL_FRACTIONAL']
                cmd = '%s %s %.7f' % (fractional_command,
                                      pseudo_head,
                                      fraction)
                os.system(cmd)

                pseudo_head += '-Fraction-%.5f' % fraction
                synth_pseudo = pseudo_head + '.psf'
                synth_block_filename = pseudo_head + '.synth'
                os.remove(name)
                shutil.copyfile(synth_pseudo, name)
                synth_block = read_vca_synth_block(
                    synth_block_filename,
                    species_number=species_number)
                synth_blocks.append(synth_block)

            if len(synth_blocks) > 0:
                f.write(format_fdf('SyntheticAtoms', list(synth_blocks)))

            label = '.'.join(np.array(name.split('.'))[:-1])
            string = '    %d %d %s' % (species_number, atomic_number, label)
            chemical_labels.append(string)
            if isinstance(spec['basis_set'], PAOBasisBlock):
                pao_basis.append(spec['basis_set'].script(label))
            else:
                basis_sizes.append(("    " + label, spec['basis_set']))
        f.write((format_fdf('ChemicalSpecieslabel', chemical_labels)))
        f.write('\n')
        f.write((format_fdf('PAO.Basis', pao_basis)))
        f.write((format_fdf('PAO.BasisSizes', basis_sizes)))
        f.write('\n')

    def pseudo_qualifier(self):
        """Get the extra string used in the middle of the pseudopotential.
        The retrieved pseudopotential for a specific element will be
        'H.xxx.psf' for the element 'H' with qualifier 'xxx'. If qualifier
        is set to None then the qualifier is set to functional name.
        """
        if self['pseudo_qualifier'] is None:
            return self['xc'][0].lower()
        else:
            return self['pseudo_qualifier']

    def read_results(self):
        """Read the results.
        """
        self.read_number_of_grid_points()
        self.read_energy()
        self.read_forces_stress()
        self.read_eigenvalues()
        self.read_kpoints()
        self.read_dipole()
        self.read_pseudo_density()
        self.read_hsx()
        self.read_dim()
        if self.results['hsx'] is not None:
            self.read_pld(self.results['hsx'].norbitals,
                          len(self.atoms))
            self.atoms.cell = self.results['pld'].cell * Bohr
        else:
            self.results['pld'] = None

        self.read_wfsx()
        self.read_ion(self.atoms)

        self.read_bands()

    def read_bands(self):
        bandpath = self['bandpath']
        if bandpath is None:
            return

        if len(bandpath.kpts) < 1:
            return

        fname = self.getpath(ext='bands')
        with open(fname) as fd:
            kpts, energies, efermi = read_bands_file(fd)
        bs = resolve_band_structure(bandpath, kpts, energies, efermi)
        self.results['bandstructure'] = bs

    def band_structure(self):
        return self.results['bandstructure']

    def read_ion(self, atoms):
        """Read the ion.xml file of each specie
        """
        from ase.calculators.siesta.import_ion_xml import get_ion

        species, species_numbers = self.species(atoms)

        self.results['ion'] = {}
        for species_number, spec in enumerate(species):
            species_number += 1

            symbol = spec['symbol']
            atomic_number = atomic_numbers[symbol]

            if spec['pseudopotential'] is None:
                if self.pseudo_qualifier() == '':
                    label = symbol
                else:
                    label = '.'.join([symbol, self.pseudo_qualifier()])
                pseudopotential = self.getpath(label, 'psf')
            else:
                pseudopotential = spec['pseudopotential']
                label = os.path.basename(pseudopotential)
                label = '.'.join(label.split('.')[:-1])

            name = os.path.basename(pseudopotential)
            name = name.split('.')
            name.insert(-1, str(species_number))
            if spec['ghost']:
                name.insert(-1, 'ghost')
                atomic_number = -atomic_number
            name = '.'.join(name)

            label = '.'.join(np.array(name.split('.'))[:-1])

            if label not in self.results['ion']:
                fname = self.getpath(label, 'ion.xml')
                if os.path.isfile(fname):
                    self.results['ion'][label] = get_ion(fname)

    def read_hsx(self):
        """
        Read the siesta HSX file.
        return a namedtuple with the following arguments:
        'norbitals', 'norbitals_sc', 'nspin', 'nonzero',
        'is_gamma', 'sc_orb2uc_orb', 'row2nnzero', 'sparse_ind2column',
        'H_sparse', 'S_sparse', 'aB2RaB_sparse', 'total_elec_charge', 'temp'
        """
        from ase.calculators.siesta.import_functions import readHSX

        filename = self.getpath(ext='HSX')
        if isfile(filename):
            self.results['hsx'] = readHSX(filename)
        else:
            self.results['hsx'] = None

    def read_dim(self):
        """
        Read the siesta DIM file
        Retrun a namedtuple with the following arguments:
        'natoms_sc', 'norbitals_sc', 'norbitals', 'nspin',
        'nnonzero', 'natoms_interacting'
        """
        from ase.calculators.siesta.import_functions import readDIM

        filename = self.getpath(ext='DIM')
        if isfile(filename):
            self.results['dim'] = readDIM(filename)
        else:
            self.results['dim'] = None

    def read_pld(self, norb, natms):
        """
        Read the siesta PLD file
        Return a namedtuple with the following arguments:
        'max_rcut', 'orb2ao', 'orb2uorb', 'orb2occ', 'atm2sp',
        'atm2shift', 'coord_sc', 'cell', 'nunit_cells'
        """
        from ase.calculators.siesta.import_functions import readPLD

        filename = self.getpath(ext='PLD')
        if isfile(filename):
            self.results['pld'] = readPLD(filename, norb, natms)
        else:
            self.results['pld'] = None

    def read_wfsx(self):
        """
        Read the siesta WFSX file
        Return a namedtuple with the following arguments:
        """
        from ase.calculators.siesta.import_functions import readWFSX

        fname_woext = os.path.join(self.directory, self.prefix)

        if isfile(fname_woext + '.WFSX'):
            filename = fname_woext + '.WFSX'
            self.results['wfsx'] = readWFSX(filename)
        elif isfile(fname_woext + '.fullBZ.WFSX'):
            filename = fname_woext + '.fullBZ.WFSX'
            readWFSX(filename)
            self.results['wfsx'] = readWFSX(filename)
        else:
            self.results['wfsx'] = None

    def read_pseudo_density(self):
        """Read the density if it is there."""
        filename = self.getpath(ext='RHO')
        if isfile(filename):
            self.results['density'] = read_rho(filename)

    def read_number_of_grid_points(self):
        """Read number of grid points from SIESTA's text-output file. """

        fname = self.getpath(ext='out')
        with open(fname, 'r') as f:
            for line in f:
                line = line.strip().lower()
                if line.startswith('initmesh: mesh ='):
                    n_points = [int(word) for word in line.split()[3:8:2]]
                    self.results['n_grid_point'] = n_points
                    break
            else:
                raise RuntimeError

    def read_energy(self):
        """Read energy from SIESTA's text-output file.
        """
        fname = self.getpath(ext='out')
        with open(fname, 'r') as f:
            text = f.read().lower()

        assert 'final energy' in text
        lines = iter(text.split('\n'))

        # Get the energy and free energy the last time it appears
        for line in lines:
            has_energy = line.startswith('siesta: etot    =')
            if has_energy:
                self.results['energy'] = float(line.split()[-1])
                line = next(lines)
                self.results['free_energy'] = float(line.split()[-1])

        if ('energy' not in self.results or
            'free_energy' not in self.results):
            raise RuntimeError

    def read_forces_stress(self):
        """Read the forces and stress from the FORCE_STRESS file.
        """
        fname = self.getpath('FORCE_STRESS')
        with open(fname, 'r') as f:
            lines = f.readlines()

        stress_lines = lines[1:4]
        stress = np.empty((3, 3))
        for i in range(3):
            line = stress_lines[i].strip().split(' ')
            line = [s for s in line if len(s) > 0]
            stress[i] = [float(s) for s in line]

        self.results['stress'] = np.array(
            [stress[0, 0], stress[1, 1], stress[2, 2],
             stress[1, 2], stress[0, 2], stress[0, 1]])

        self.results['stress'] *= Ry / Bohr**3

        start = 5
        self.results['forces'] = np.zeros((len(lines) - start, 3), float)
        for i in range(start, len(lines)):
            line = [s for s in lines[i].strip().split(' ') if len(s) > 0]
            self.results['forces'][i - start] = [float(s) for s in line[2:5]]

        self.results['forces'] *= Ry / Bohr

    def read_eigenvalues(self):
        """ A robust procedure using the suggestion by Federico Marchesin """

        fname = self.getpath(ext='EIG')
        try:
            with open(fname, "r") as f:
                self.results['fermi_energy'] = float(f.readline())
                n, nspin, nkp = map(int, f.readline().split())
                _ee = np.split(
                    np.array(f.read().split()).astype(np.float), nkp)
        except (IOError):
            return 1

        ksn2e = np.delete(_ee, 0, 1).reshape([nkp, nspin, n])

        eigarray = np.empty((nspin, nkp, n))
        eigarray[:] = np.inf

        for k, sn2e in enumerate(ksn2e):
            for s, n2e in enumerate(sn2e):
                eigarray[s, k, :] = n2e

        assert np.isfinite(eigarray).all()

        self.results['eigenvalues'] = eigarray
        return 0

    def read_kpoints(self):
        """ Reader of the .KP files """

        fname = self.getpath(ext='KP')
        try:
            with open(fname, "r") as fd:
                nkp = int(next(fd))
                kpoints = np.empty((nkp, 3))
                kweights = np.empty(nkp)

                for i in range(nkp):
                    line = next(fd)
                    tokens = line.split()
                    numbers = np.array(tokens[1:]).astype(float)
                    kpoints[i] = numbers[:3]
                    kweights[i] = numbers[3]

        except (IOError):
            return 1

        self.results['kpoints'] = kpoints
        self.results['kweights'] = kweights

        return 0

    def read_dipole(self):
        """Read dipole moment. """
        dipole = np.zeros([1, 3])
        with open(self.getpath(ext='out'), 'r') as f:
            for line in f:
                if line.rfind('Electric dipole (Debye)') > -1:
                    dipole = np.array([float(f) for f in line.split()[5:8]])
        # debye to e*Ang
        self.results['dipole'] = dipole * 0.2081943482534

    def pyscf_tddft(
            self,
            Edir=np.array([1.0, 0.0, 0.0]),
            freq=np.arange(0.0, 10.0, 0.1),
            units='au',
            run_tddft=True,
            save_kernel=True,
            kernel_name="tddft_kernel.npy",
            fname="pol_tensor.npy",
            fname_nonin="noninpol_tensor.npy", **kw):
        """
        Perform TDDFT calculation using the pyscf.nao module for a molecule.

        Parameters
        ----------
        freq: array like
            frequency range for which the polarizability should
            be computed, in eV
        units : str, optional
            unit for the returned polarizability, can be au (atomic units)
            or nm**2
        run_tddft: to run the tddft_calculation or not
        fname: str
            Name of input file name for polariazbility tensor.
            if run_tddft is True: output file
            if run_tddft is False: input file

        kw: keywords for the tddft_iter function from pyscf

        Returns
        -------
            Add to the self.results dict the following items:
        freq range: array like
            array of dimension (nff) containing the frequency range in eV.

        polarizability nonin: array like (complex)
            array of dimension (nff, 3, 3) with nff the frequency number,
            the second and third dimension are the matrix elements of the
            non-interactive polarizability::

                P_xx, P_xy, P_xz, Pyx, .......


        polarizability: array like (complex)
            array of dimension (nff, 3, 3) with nff the frequency number,
            the second and third dimension are the matrix elements of the
            interactive polarizability::

                P_xx, P_xy, P_xz, Pyx, .......

        density change nonin: array like (complex)
            contains the non interacting density change in product basis

        density change inter: array like (complex)
            contains the interacting density change in product basis

        References
        ----------
        https://github.com/cfm-mpc/pyscf/tree/nao

        Example
        -------
        from ase.units import Ry, eV, Ha
        from ase.calculators.siesta import Siesta
        from ase import Atoms
        import numpy as np
        import matplotlib.pyplot as plt

        # Define the systems
        Na8 = Atoms('Na8',
                     positions=[[-1.90503810, 1.56107288, 0.00000000],
                                [1.90503810, 1.56107288, 0.00000000],
                                [1.90503810, -1.56107288, 0.00000000],
                                [-1.90503810, -1.56107288, 0.00000000],
                                [0.00000000, 0.00000000, 2.08495836],
                                [0.00000000, 0.00000000, -2.08495836],
                                [0.00000000, 3.22798122, 2.08495836],
                                [0.00000000, 3.22798122, -2.08495836]],
                     cell=[20, 20, 20])

        # Siesta input
        siesta = Siesta(
                    mesh_cutoff=150 * Ry,
                    basis_set='DZP',
                    pseudo_qualifier='',
                    energy_shift=(10 * 10**-3) * eV,
                    fdf_arguments={
                        'SCFMustConverge': False,
                        'COOP.Write': True,
                        'WriteDenchar': True,
                        'PAO.BasisType': 'split',
                        'DM.Tolerance': 1e-4,
                        'DM.MixingWeight': 0.01,
                        'MaxSCFIterations': 300,
                        'DM.NumberPulay': 4,
                        'XML.Write': True})

        Na8.set_calculator(siesta)
        e = Na8.get_potential_energy()
        freq, pol = siesta.get_polarizability_pyscf_inter(
            label="siesta",
            jcutoff=7,
            iter_broadening=0.15/Ha,
            xc_code='LDA,PZ',
            tol_loc=1e-6,
            tol_biloc=1e-7,
            freq = np.arange(0.0, 5.0, 0.05))
        # plot polarizability
        plt.plot(freq, pol[:, 0, 0].imag)
        plt.show()
        """

        from ase.calculators.siesta.mbpt_lcao_utils import pol2cross_sec
        assert units in ["nm**2", "au"]

        if run_tddft:
            from pyscf.nao import tddft_iter
            from ase.units import Ha

            tddft = tddft_iter(**kw)
            if save_kernel:
                np.save(kernel_name, tddft.kernel)

            omegas = freq / Ha + 1j * tddft.eps
            tddft.comp_dens_nonin_along_Eext(omegas, Eext=Edir)
            tddft.comp_dens_inter_along_Eext(omegas, Eext=Edir)

            # save polarizability tensor and density change to files
            self.results["freq range"] = freq
            self.results['polarizability nonin'] = np.zeros(
                (freq.size, 3, 3),
                dtype=tddft.p0_mat.dtype)
            self.results['polarizability inter'] = np.zeros(
                (freq.size, 3, 3),
                dtype=tddft.p_mat.dtype)
            self.results["density change nonin"] = tddft.dn0
            self.results["density change inter"] = tddft.dn
            for xyz1 in range(3):
                for xyz2 in range(3):
                    if units == 'nm**2':
                        p0 = pol2cross_sec(-tddft.p0_mat[xyz1, xyz2, :], freq)
                        p = pol2cross_sec(-tddft.p_mat[xyz1, xyz2, :], freq)
                        self.results['polarizability nonin'][:, xyz1, xyz2] = \
                            p0
                        self.results['polarizability inter'][:, xyz1, xyz2] = \
                            p
                    else:
                        self.results['polarizability nonin'][:, xyz1, xyz2] = \
                            -tddft.p0_mat[xyz1, xyz2, :]
                        self.results['polarizability inter'][:, xyz1, xyz2] = \
                            -tddft.p_mat[xyz1, xyz2, :]

        else:
            # load polarizability tensor from previous calculations
            p0_mat = np.load(fname_nonin)
            p_mat = np.load(fname)

            self.results['polarizability nonin'] = np.zeros(
                (freq.size, 3, 3),
                dtype=p0_mat.dtype)
            self.results['polarizability inter'] = np.zeros(
                (freq.size, 3, 3),
                dtype=p_mat.dtype)

            for xyz1 in range(3):
                for xyz2 in range(3):
                    if units == 'nm**2':
                        p0 = pol2cross_sec(-p0_mat[xyz1, xyz2, :], freq)
                        p = pol2cross_sec(-p_mat[xyz1, xyz2, :], freq)

                        self.results['polarizability nonin'][:, xyz1, xyz2] = \
                            p0
                        self.results['polarizability inter'][:, xyz1, xyz2] = \
                            p
                    else:
                        self.results['polarizability nonin'][:, xyz1, xyz2] = \
                            -p0_mat[xyz1, xyz2, :]
                        self.results['polarizability inter'][:, xyz1, xyz2] = \
                            -p_mat[xyz1, xyz2, :]

    def pyscf_tddft_eels(
            self,
            velec=np.array([20.0, 0.0, 0.0]),
            b=np.array([0.0, 0.0, 0.0]),
            freq=np.arange(0.0, 10.0, 0.1),
            tddft=None,
            save_kernel=True,
            kernel_name="tddft_kernel.npy",
            tmp_fname=None,
            **kw):
        r"""
        Perform TDDFT calculation using the pyscf.nao module for a molecule.
        The external pertubation is created by a electron moving at
        the velocity velec and with an impact parameter b.

        Parameters
        ----------
        freq: array like
            frequency range for which the polarizability should
            be computed, in eV
        velec: array like
            velocity vector of the projectile
        b: array like
            offset vector of the projectile
        tddft: tddft_tem class from a previous calculation
        save_kernel: save the kernel for future use
        kernel_name: name of the file for the kernel
        tmp_fname: temporary name to save the eels spectra
        while running the calculations.
        kw: keywords for the tddft_tem function from pyscf

        Returns
        -------
        tddft:
            if running pyscf_tddft_eels in a loop over the velocity or the
            impact parameter, there is no point to initialize again the tddft
            calculation (vertex and kernel will be the same)

            Add to the self.results dict the following items:
        freq range: array like
            array of dimension (nff) containing the frequency range in eV.

        eel spectra nonin: array like (complex)
            array of dimension (nff) with nff the frequency number,


        eel spectra inter: array like (complex)
            array of dimension (nff) with nff the frequency number,

        density change eels nonin: array like (complex)
            contains the non interacting density change in product basis

        density change eels inter: array like (complex)
            contains the interacting density change in product basis

        References
        ----------
        https://github.com/cfm-mpc/pyscf/tree/nao

        Example
        -------
        from ase.units import Ry, eV, Ha
        from ase.calculators.siesta import Siesta
        from ase import Atoms
        import numpy as np
        import matplotlib.pyplot as plt

        # Define the systems
        Na8 = Atoms('Na8',
                    positions=[[-1.90503810, 1.56107288, 0.00000000],
                               [1.90503810, 1.56107288, 0.00000000],
                               [1.90503810, -1.56107288, 0.00000000],
                               [-1.90503810, -1.56107288, 0.00000000],
                               [0.00000000, 0.00000000, 2.08495836],
                               [0.00000000, 0.00000000, -2.08495836],
                               [0.00000000, 3.22798122, 2.08495836],
                               [0.00000000, 3.22798122, -2.08495836]],
                    cell=[20, 20, 20])

        # enter siesta input
        siesta = Siesta(
            mesh_cutoff=150 * Ry,
            basis_set='DZP',
            pseudo_qualifier='',
            energy_shift=(10 * 10**-3) * eV,
            fdf_arguments={
                'SCFMustConverge': False,
                'COOP.Write': True,
                'WriteDenchar': True,
                'PAO.BasisType': 'split',
                'DM.Tolerance': 1e-4,
                'DM.MixingWeight': 0.01,
                'MaxSCFIterations': 300,
                'DM.NumberPulay': 4,
                'XML.Write': True})


        Na8.set_calculator(siesta)
        e = Na8.get_potential_energy()
        tddft = siesta.pyscf_tddft_eels(
            label="siesta", jcutoff=7, iter_broadening=0.15/Ha,
            xc_code='LDA,PZ', tol_loc=1e-6, tol_biloc=1e-7,
            freq=np.arange(0.0, 5.0, 0.05))

        # plot eel spectra
        fig = plt.figure(1)
        ax1 = fig.add_subplot(121)
        ax2 = fig.add_subplot(122)
        ax1.plot(siesta.results["freq range"],
                 siesta.results["eel spectra nonin"].imag)
        ax2.plot(siesta.results["freq range"],
                 siesta.results["eel spectra inter"].imag)

        ax1.set_xlabel(r"$\omega$ (eV)")
        ax2.set_xlabel(r"$\omega$ (eV)")

        ax1.set_ylabel(r"Im($P_{xx}$) (au)")
        ax2.set_ylabel(r"Im($P_{xx}$) (au)")

        ax1.set_title(r"Non interacting")
        ax2.set_title(r"Interacting")

        fig.tight_layout()
        plt.show()
        """
        from pyscf.nao import tddft_tem
        from ase.units import Ha

        assert velec.size == 3
        assert b.size == 3

        if tddft is None:
            self.results["freq range"] = freq
            omegas = freq / Ha

            # for eels, omega is real array
            tddft = tddft_tem(freq=omegas, **kw)
            if save_kernel:
                np.save(kernel_name, tddft.kernel)

        self.results['eel spectra nonin'] = \
            tddft.get_spectrum_nonin(velec=velec,
                                     beam_offset=b,
                                     tmp_fname=tmp_fname)

        self.results['eel spectra inter'] = \
            tddft.get_spectrum_inter(velec=velec,
                                     beam_offset=b,
                                     tmp_fname=tmp_fname)

        self.results["density change eels nonin"] = tddft.dn0
        self.results["density change eels inter"] = tddft.dn

        return tddft

    def get_polarizability_mbpt(self, mbpt_inp=None,
                                output_name='mbpt_lcao.out',
                                format_output='hdf5', units='au'):
        """
        Warning!!
            Out dated version, try get_polarizability_pyscf


        Calculate the polarizability by running the mbpt_lcao program.
        The mbpt_lcao program need the siesta output, therefore siesta need
        to be run first.

        Parameters
        ----------
        mbpt_inp : dict, optional
            dictionnary of the input for the mbpt_lcao program
            (http://mbpt-domiprod.wikidot.com/list-of-parameters)
            if mbpt_inp is None, the function read the output file
            from a previous mbpt_lcao run.
        output_name : str, optional
            Name of the mbpt_lcao output
        format_output : str, optional
            Format of the mbpt_lcao output data,
            if hdf5, the output name is tddft_iter_output.hdf5 if
            do_tddft_iter is set to 1 the output name is
            tddft_tem_output.hdf5 if do_tddft_tem is set to 1
            if txt, a lot of output data files are produced depending on
            the input, in the text and fortran binaries format
        units : str, optional
            unit for the returned polarizability, can be au (atomic units)
            or nm**2

        Returns
        -------
        freq : array like
            array of dimension (nff) containing the frequency range in eV.

        self.results['polarizability'], array like
            array of dimension (nff, 3, 3) with nff the frequency number,
            the second and third dimension are the matrix elements of the
            polarizability::

                P_xx, P_xy, P_xz, Pyx, .......

        References
        ----------
        http://mbpt-domiprod.wikidot.com

        Example
        -------
        import os
        from ase.units import Ry, eV
        from ase.calculators.siesta import Siesta
        from ase import Atoms
        import numpy as np
        import matplotlib.pyplot as plt

        #Define the systems
        Na8 = Atoms('Na8',
        positions=[[-1.90503810, 1.56107288, 0.00000000],
                    [1.90503810, 1.56107288, 0.00000000],
                    [1.90503810, -1.56107288, 0.00000000],
                    [-1.90503810, -1.56107288, 0.00000000],
                    [0.00000000, 0.00000000, 2.08495836],
                    [0.00000000, 0.00000000, -2.08495836],
                    [0.00000000, 3.22798122, 2.08495836],
                    [0.00000000, 3.22798122, -2.08495836]],
                    cell=[20, 20, 20])

        #enter siesta input
        siesta = Siesta(
            mesh_cutoff=150 * Ry,
            basis_set='DZP',
            pseudo_qualifier='',
            energy_shift=(10 * 10**-3) * eV,
            fdf_arguments={
            'SCFMustConverge': False,
            'COOP.Write': True,
            'WriteDenchar': True,
            'PAO.BasisType': 'split',
            'DM.Tolerance': 1e-4,
            'DM.MixingWeight': 0.01,
            'MaxSCFIterations': 300,
            'DM.NumberPulay': 4})


        #mbpt_lcao input
        mbpt_inp = {'prod_basis_type' : 'MIXED',
                    'solver_type' : 1,
                    'gmres_eps' : 0.001,
                    'gmres_itermax':256,
                    'gmres_restart':250,
                    'gmres_verbose':20,
                    'xc_ord_lebedev':14,
                    'xc_ord_gl':48,
                    'nr':512,
                    'akmx':100,
                    'eigmin_local':1e-06,
                    'eigmin_bilocal':1e-08,
                    'freq_eps_win1':0.15,
                    'd_omega_win1':0.05,
                    'dt':0.1,
                    'omega_max_win1':5.0,
                    'ext_field_direction':2,
                    'dr':np.array([0.3, 0.3, 0.3]),
                    'para_type':'MATRIX',
                    'chi0_v_algorithm':14,
                    'format_output':'text',
                    'comp_dens_chng_and_polarizability':1,
                    'store_dens_chng':1,
                    'enh_given_volume_and_freq':0,
                    'diag_hs':0,
                    'do_tddft_tem':0,
                    'do_tddft_iter':1,
                    'plot_freq':3.02,
                    'gwa_initialization':'SIESTA_PB'}


        Na8.set_calculator(siesta)
        e = Na8.get_potential_energy() #run siesta
        freq, pol = siesta.get_polarizability_siesta(mbpt_inp,
                                                     format_output='txt',
                                                     units='nm**2')

        #plot polarizability
        plt.plot(freq, pol[:, 0, 0])

        plt.show()
        """
        from ase.calculators.siesta.mbpt_lcao import MBPT_LCAO
        from ase.calculators.siesta.mbpt_lcao_io import read_mbpt_lcao_output

        warnings.warn("Out dated version, try get_polarizability_pyscf")

        if mbpt_inp is not None:
            tddft = MBPT_LCAO(mbpt_inp)
            tddft.run_mbpt_lcao(output_name, True)

        r = read_mbpt_lcao_output()

        r.args.format_input = format_output

        # read real part
        r.args.ReIm = 're'
        data = r.Read()
        self.results['polarizability'] = data.Array

        # read imaginary part
        r.args.ReIm = 'im'
        data = r.Read()
        self.results['polarizability'] = (self.results['polarizability'] +
                                          complex(0.0, 1.0) * data.Array)

        if units == 'nm**2':
            from ase.calculators.siesta.mbpt_lcao_utils import pol2cross_sec
            for i in range(2):
                for j in range(2):
                    p = pol2cross_sec(self.results['polarizability'][:, i, j],
                                      data.freq)
                    self.results['polarizability'][:, i, j] = p

            print('unit nm**2')
            # self.results['polarizability'] = data.Array
        elif units == 'au':
            print('unit au')
            # self.results['polarizability'] = data.Array
        else:
            raise ValueError('units can be only au or nm**2')

        return data.freq, self.results['polarizability']

    def get_fermi_level(self):
        return self.results['fermi_energy']

    def get_k_point_weights(self):
        return self.results['kweights']

    def get_ibz_k_points(self):
        return self.results['kpoints']


class Siesta3_2(Siesta):
    def __init__(self, *args, **kwargs):
        warnings.warn(
            "The Siesta3_2 calculator class will no longer be supported. "
            "Use 'ase.calculators.siesta.Siesta in stead. "
            "If using the ASE interface with SIESTA 3.2 you must explicitly "
            "include the keywords 'SpinPolarized', 'NonCollinearSpin' and "
            "'SpinOrbit' if needed.",
            np.VisibleDeprecationWarning)
        Siesta.__init__(self, *args, **kwargs)



from ase import Atoms
from ase.io import Trajectory
from gpaw import GPAW

atoms = Atoms('N2', positions=[[0, 0, -1], [0, 0, 1]])
atoms.center(vacuum=3.0)

calc = GPAW(mode='lcao', basis='dzp', txt='gpaw.txt')
atoms.calc = calc

traj = Trajectory('binding_curve.traj', 'w')

step = 0.05
nsteps = int(3 / step)

for i in range(nsteps):
    d = 0.5 + i * step
    atoms.positions[1, 2] = atoms.positions[0, 2] + d
    atoms.center(vacuum=3.0)
    e = atoms.get_potential_energy()
    f = atoms.get_forces()
    print('distance, energy', d, e)
    print('force', f)
    traj.write(atoms)




import traceback

import numpy as np

from ase import Atoms

# This module the InterfaceTester class which tests the extent to
# which an object behaves like an ASE calculator.
#
# It runs the ASE interface methods and performs a few very basic checks
# on the returned objects, then writes a list of errors.
#
# Future improvements: Check that arrays are padded correctly, verify
# more information about shapes, maybe do some complicated state
# changes and check that the calculator behaves properly.


class Args:
    # This class is just syntantical sugar to pass args and kwargs
    # easily when testing many methods after one another.
    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs

    def unpack(self):
        return self.args, self.kwargs


args = Args


class Error:
    def __init__(self, code, error, methname):
        self.code = code
        self.error = error
        self.methname = methname
        self.txt = traceback.format_exc()
        self.callstring = None


class InterfaceChecker:
    def __init__(self, obj):
        self.obj = obj
        self.returnvalues = {}
        self.errors = []

    def _check(self, methname, args=args(), rtype=None):
        args, kwargs = args.unpack()

        try:
            meth = getattr(self.obj, methname)
        except AttributeError as err:
            return Error('MISSING', err, methname)

        try:
            value = meth(*args, **kwargs)
        except NotImplementedError as err:
            return Error('not implemented', err, methname)
        except Exception as err:
            return Error(err.__class__.__name__, err, methname)
        else:
            self.returnvalues[methname] = value

        if rtype is not None:
            if not isinstance(value, rtype):
                return Error('TYPE', TypeError('got %s but expected %s'
                                               % (type(value), rtype)),
                             methname)
        return None

    def check(self, methname, args=args(), rtype=None):
        pargs, kwargs = args.unpack()
        
        def get_string_repr(obj):
            if isinstance(obj, Atoms):
                return '<Atoms>'
            else:
                return repr(obj)

        pargsstrs = [get_string_repr(obj) for obj in pargs]
        kwargsstrs = ['%s=%s' % (key, get_string_repr(kwargs[key]))
                      for key in sorted(kwargs)]
        pargskwargsstr = ', '.join(pargsstrs + kwargsstrs)
        err = self._check(methname, args, rtype)
        callstring = '%s(%s)' % (methname, pargskwargsstr)
        if err is None:
            status = 'ok'
        else:
            status = err.code
            err.callstring = callstring
            self.errors.append(err)
        print('%16s : %s' % (status, callstring))


def check_interface(calc):
    tester = InterfaceChecker(calc)
    c = tester.check

    system = calc.get_atoms()

    # Methods specified by ase.calculators.interface.Calculator
    c('get_atoms', rtype=Atoms)
    c('get_potential_energy', rtype=float)
    c('get_potential_energy', args(atoms=system), rtype=float)
    c('get_potential_energy', args(atoms=system, force_consistent=True),
      rtype=float)
    c('get_forces', args(system), np.ndarray)
    c('get_stress', args(system), np.ndarray)
    c('calculation_required', args(system, []), rtype=bool)

    # Methods specified by ase.calculators.interface.DFTCalculator
    c('get_number_of_bands', rtype=int)
    c('get_xc_functional', rtype=str)
    c('get_bz_k_points', rtype=np.ndarray)
    c('get_number_of_spins', rtype=int)
    c('get_spin_polarized', rtype=bool)
    c('get_ibz_k_points', rtype=np.ndarray)
    c('get_k_point_weights', rtype=np.ndarray)

    for meth in ['get_pseudo_density', 'get_effective_potential']:
        c(meth, rtype=np.ndarray)
        c(meth, args(spin=0, pad=False), rtype=np.ndarray)
        spinpol = tester.returnvalues.get('get_spin_polarized')
        if spinpol:
            c(meth, args(spin=1, pad=True), rtype=np.ndarray)

    for pad in [False, True]:
        c('get_pseudo_density', args(spin=None, pad=pad), rtype=np.ndarray)

    for meth in ['get_pseudo_density', 'get_effective_potential']:
        c(meth, args(spin=0, pad=False), rtype=np.ndarray)
        spinpol = tester.returnvalues.get('get_spin_polarized')
        if spinpol:
            c(meth, args(spin=1, pad=True), rtype=np.ndarray)

    nbands = tester.returnvalues.get('get_number_of_bands')
    if nbands is not None and isinstance(nbands, int) and nbands > 0:
        c('get_pseudo_wave_function', args(band=nbands - 1), rtype=np.ndarray)
        c('get_pseudo_wave_function',
          args(band=nbands - 1, kpt=0, spin=0, broadcast=False, pad=False),
          rtype=np.ndarray)
    c('get_eigenvalues', args(kpt=0, spin=0), rtype=np.ndarray)
    c('get_occupation_numbers', args(kpt=0, spin=0), rtype=np.ndarray)
    c('get_fermi_level', rtype=float)
    # c('initial_wanner', ........) what the heck?
    # c('get_wannier_localization_matrix', ...)  No.
    c('get_magnetic_moment', args(atoms=system), rtype=float)
    # c('get_number_of_grid_points', rtype=tuple) # Hmmmm.  Not for now...

    # Optional methods sometimes invoked by ase.atoms.Atoms
    c('get_magnetic_moments', rtype=np.ndarray)
    c('get_charges', rtype=np.ndarray)
    c('get_potential_energies', rtype=np.ndarray)
    c('get_stresses', rtype=np.ndarray)
    c('get_dipole_moment', rtype=np.ndarray)

    real_errs = [err for err in tester.errors
                 if not isinstance(err.error, NotImplementedError)]
    if len(real_errs) > 0:
        print()
        print('Errors')
        print('======')
        for err in tester.errors:
            print('%s: %s' % (err.code, err.callstring))
            print(err.txt)
            print()

    return tester.errors


def main_gpaw():
    from gpaw import GPAW
    from ase.build import molecule
    system = molecule('H2')
    system.center(vacuum=1.5)
    system.pbc = 1
    calc = GPAW(h=0.3, mode='lcao', txt=None)
    system.set_calculator(calc)
    system.get_potential_energy()
    check_interface(calc)


def main_octopus():
    from octopus import Octopus
    from ase.build import molecule
    system = molecule('H2')
    system.center(vacuum=1.5)
    system.pbc = 1
    calc = Octopus()
    system.set_calculator(calc)
    system.get_potential_energy()
    check_interface(calc)
    

if __name__ == '__main__':
    main_gpaw()



from ase.constraints import FixAtoms, FixBondLength
from ase.build import molecule, bulk
from ase.io.trajectory import Trajectory, get_header_data
from ase.io import read

a0 = molecule('H2O')
a1 = a0.copy()
a1.rattle(stdev=0.5)
a2 = a0.copy()
a2.set_masses()
a2.center(vacuum=2.0)
a2.rattle(stdev=0.2)
a3 = molecule('CH3CH2OH')
a4 = bulk('Au').repeat((2, 2, 2))
a5 = bulk('Cu').repeat((2, 2, 3))

# Add constraints to some of the images:
images = [a0, a1, a2, a3, a4, a5]
for i, img in enumerate(images[3:]):
    img.set_constraint(FixAtoms(indices=range(i + 3)))
    if i == 2:
        img.constraints.append(FixBondLength(5, 6))

traj = Trajectory('out.traj', 'w')
for i, img in enumerate(images):
    traj.write(img)
    print(i, traj.multiple_headers)
    assert traj.multiple_headers == (i >= 2)
traj.close()

rtraj = Trajectory('out.traj')
newimages = list(rtraj)

assert len(images) == len(newimages)
for i in range(len(images)):
    assert images[i] == newimages[i], i
    h1 = get_header_data(images[i])
    h2 = get_header_data(newimages[i])
    print(i, images[i])
    print(h1)
    print(h2)
    print()
    # assert headers_equal(h1, h2)

# Test append mode:
with Trajectory('out.traj', 'a') as atraj:
    atraj.write(molecule('H2'))
    atraj.write(molecule('H2'))
read('out.traj@:')



# flake8: noqa
"""This module defines an ASE interface to ABINIT.

http://www.abinit.org/
"""

import os
from glob import glob
from os.path import join

import numpy as np

from ase.units import Bohr, Hartree, fs
from ase.data import chemical_symbols
from ase.io.abinit import read_abinit
from ase.calculators.calculator import FileIOCalculator, Parameters, kpts2mp, \
    ReadError


keys_with_units = {
    'toldfe': 'eV',
    'tsmear': 'eV',
    'paoenergyshift': 'eV',
    'zmunitslength': 'Bohr',
    'zmunitsangle': 'rad',
    'zmforcetollength': 'eV/Ang',
    'zmforcetolangle': 'eV/rad',
    'zmmaxdispllength': 'Ang',
    'zmmaxdisplangle': 'rad',
    'ecut': 'eV',
    'pawecutdg': 'eV',
    'dmenergytolerance': 'eV',
    'electronictemperature': 'eV',
    'oneta': 'eV',
    'onetaalpha': 'eV',
    'onetabeta': 'eV',
    'onrclwf': 'Ang',
    'onchemicalpotentialrc': 'Ang',
    'onchemicalpotentialtemperature': 'eV',
    'mdmaxcgdispl': 'Ang',
    'mdmaxforcetol': 'eV/Ang',
    'mdmaxstresstol': 'eV/Ang**3',
    'mdlengthtimestep': 'fs',
    'mdinitialtemperature': 'eV',
    'mdtargettemperature': 'eV',
    'mdtargetpressure': 'eV/Ang**3',
    'mdnosemass': 'eV*fs**2',
    'mdparrinellorahmanmass': 'eV*fs**2',
    'mdtaurelax': 'fs',
    'mdbulkmodulus': 'eV/Ang**3',
    'mdfcdispl': 'Ang',
    'warningminimumatomicdistance': 'Ang',
    'rcspatial': 'Ang',
    'kgridcutoff': 'Ang',
    'latticeconstant': 'Ang'}


class Abinit(FileIOCalculator):
    """Class for doing ABINIT calculations.

    The default parameters are very close to those that the ABINIT
    Fortran code would use.  These are the exceptions::

      calc = Abinit(label='abinit', xc='LDA', ecut=400, toldfe=1e-5)
    """

    implemented_properties = ['energy', 'forces', 'stress', 'magmom']
    command = 'abinit < PREFIX.files > PREFIX.log'

    default_parameters = dict(
        xc='LDA',
        smearing=None,
        kpts=None,
        charge=0.0,
        raw=None,
        pps='fhi')

    def __init__(self, restart=None, ignore_bad_restart_file=False,
                 label='abinit', atoms=None, scratch=None, **kwargs):
        """Construct ABINIT-calculator object.

        Parameters
        ==========
        label: str
            Prefix to use for filenames (label.in, label.txt, ...).
            Default is 'abinit'.

        Examples
        ========
        Use default values:

        >>> h = Atoms('H', calculator=Abinit(ecut=200, toldfe=0.001))
        >>> h.center(vacuum=3.0)
        >>> e = h.get_potential_energy()

        """

        self.scratch = scratch

        self.species = []
        self.ppp_list = []

        FileIOCalculator.__init__(self, restart, ignore_bad_restart_file,
                                  label, atoms, **kwargs)

    def check_state(self, atoms):
        system_changes = FileIOCalculator.check_state(self, atoms)
        # Ignore boundary conditions:
        if 'pbc' in system_changes:
            system_changes.remove('pbc')
        return system_changes

    def set(self, **kwargs):
        changed_parameters = FileIOCalculator.set(self, **kwargs)
        if changed_parameters:
            self.reset()

    def write_input(self, atoms, properties=tuple(), system_changes=tuple(),
        raise_exception=True):
        """Write input parameters to files-file."""

        FileIOCalculator.write_input(self, atoms, properties, system_changes)

        try:
            if ('numbers' in system_changes or
                'initial_magmoms' in system_changes):
                self.initialize(atoms, raise_exception=raise_exception)
        except Exception as e:
            print(e, '... but I continue to complete the abinit.in')
            pass

        fh = open(self.label + '.files', 'w')

        fh.write('%s\n' % (self.prefix + '.in'))  # input
        fh.write('%s\n' % (self.prefix + '.txt'))  # output
        fh.write('%s\n' % (self.prefix + 'i'))  # input
        fh.write('%s\n' % (self.prefix + 'o'))  # output
        
        # XXX:
        # scratch files
        #scratch = self.scratch
        #if scratch is None:
        #    scratch = dir
        #if not os.path.exists(scratch):
        #    os.makedirs(scratch)
        #fh.write('%s\n' % (os.path.join(scratch, prefix + '.abinit')))
        fh.write('%s\n' % (self.prefix + '.abinit'))
        # Provide the psp files
        for ppp in self.ppp_list:
            fh.write('%s\n' % (ppp)) # psp file path

        fh.close()

        # Abinit will write to label.txtA if label.txt already exists,
        # so we remove it if it's there:
        filename = self.label + '.txt'
        if os.path.isfile(filename):
            os.remove(filename)

        param = self.parameters
        param.write(self.label + '.ase')

        fh = open(self.label + '.in', 'w')
        inp = {}
        inp.update(param)
        for key in ['xc', 'smearing', 'kpts', 'pps', 'raw']:
            del inp[key]

        smearing = param.get('smearing')
        if 'tsmear' in param or 'occopt' in param:
            assert smearing is None

        if smearing is not None:
            inp['occopt'] = {'fermi-dirac': 3,
                             'gaussian': 7}[smearing[0].lower()]
            inp['tsmear'] = smearing[1]

        inp['natom'] = len(atoms)

        if 'nbands' in param:
            inp['nband'] = param.nbands
            del inp['nbands']

        # ixc is set from paw/xml file. Ignore 'xc' setting then.
        if param.get('pps') not in ['pawxml']:
            if 'ixc' not in param:
                inp['ixc'] = {'LDA': 7,
                              'PBE': 11,
                              'revPBE': 14,
                              'RPBE': 15,
                              'WC': 23}[param.xc]

        magmoms = atoms.get_initial_magnetic_moments()
        if magmoms.any():
            inp['nsppol'] = 2
            fh.write('spinat\n')
            for n, M in enumerate(magmoms):
                fh.write('%.14f %.14f %.14f\n' % (0, 0, M))
        else:
            inp['nsppol'] = 1

        for key in sorted(inp.keys()):
            value = inp[key]
            unit = keys_with_units.get(key)
            if unit is None:
                fh.write('%s %s\n' % (key, value))
            else:
                if 'fs**2' in unit:
                    value /= fs**2
                elif 'fs' in unit:
                    value /= fs
                fh.write('%s %e %s\n' % (key, value, unit))

        if param.raw is not None:
            for line in param.raw:
                if isinstance(line, tuple):
                    fh.write(' '.join(['%s' % x for x in line]) + '\n')
                else:
                    fh.write('%s\n' % line)

        fh.write('#Definition of the unit cell\n')
        fh.write('acell\n')
        fh.write('%.14f %.14f %.14f Angstrom\n' % (1.0, 1.0, 1.0))
        fh.write('rprim\n')
        if atoms.number_of_lattice_vectors != 3:
            raise RuntimeError('Abinit requires a 3D cell, but cell is {}'
                               .format(atoms.cell))
        for v in atoms.cell:
            fh.write('%.14f %.14f %.14f\n' %  tuple(v))

        fh.write('chkprim 0 # Allow non-primitive cells\n')

        fh.write('#Definition of the atom types\n')
        fh.write('ntypat %d\n' % (len(self.species)))
        fh.write('znucl')
        for n, Z in enumerate(self.species):
            fh.write(' %d' % (Z))
        fh.write('\n')
        fh.write('#Enumerate different atomic species\n')
        fh.write('typat')
        fh.write('\n')
        self.types = []
        for Z in atoms.numbers:
            for n, Zs in enumerate(self.species):
                if Z == Zs:
                    self.types.append(n + 1)
        n_entries_int = 20  # integer entries per line
        for n, type in enumerate(self.types):
            fh.write(' %d' % (type))
            if n > 1 and ((n % n_entries_int) == 1):
                fh.write('\n')
        fh.write('\n')

        fh.write('#Definition of the atoms\n')
        fh.write('xangst\n')
        for pos in atoms.positions:
            fh.write('%.14f %.14f %.14f\n' %  tuple(pos))

        if 'kptopt' not in param:
            mp = kpts2mp(atoms, param.kpts)
            fh.write('kptopt 1\n')
            fh.write('ngkpt %d %d %d\n' % tuple(mp))
            fh.write('nshiftk 1\n')
            fh.write('shiftk\n')
            fh.write('%.1f %.1f %.1f\n' % tuple((np.array(mp) + 1) % 2 * 0.5))

        fh.write('chkexit 1 # abinit.exit file in the running directory terminates after the current SCF\n')

        fh.close()

    def read(self, label):
        """Read results from ABINIT's text-output file."""
        FileIOCalculator.read(self, label)
        filename = self.label + '.txt'
        if not os.path.isfile(filename):
            raise ReadError('ABINIT output file '+filename+' is missing.')

        self.atoms = read_abinit(self.label + '.in')
        self.parameters = Parameters.read(self.label + '.ase')

        self.initialize(self.atoms)
        self.read_results()

    def read_results(self):
        filename = self.label + '.txt'
        text = open(filename).read().lower()

        for line in iter(text.split('\n')):
            if line.rfind('error') > -1 or line.rfind('was not enough scf cycles to converge') > -1:
                raise ReadError(line)
            if line.rfind('natom  ') > -1:
                natoms = int(line.split()[-1])

        lines = iter(text.split('\n'))
        # Stress:
        # Printed in the output in the following format [Hartree/Bohr^3]:
        # sigma(1 1)=  4.02063464E-04  sigma(3 2)=  0.00000000E+00
        # sigma(2 2)=  4.02063464E-04  sigma(3 1)=  0.00000000E+00
        # sigma(3 3)=  4.02063464E-04  sigma(2 1)=  0.00000000E+00
        for line in lines:
            if line.rfind(
                'cartesian components of stress tensor (hartree/bohr^3)') > -1:
                stress = np.empty(6)
                for i in range(3):
                    entries = next(lines).split()
                    stress[i] = float(entries[2])
                    stress[i + 3] = float(entries[5])
                self.results['stress'] = stress * Hartree / Bohr**3
                break
        else:
            raise RuntimeError

        # Energy [Hartree]:
        # Warning: Etotal could mean both electronic energy and free energy!
        etotal = None
        efree = None
        if 'PAW method is used'.lower() in text:  # read DC energy according to M. Torrent
            for line in iter(text.split('\n')):
                if line.rfind('>>>>> internal e=') > -1:
                    etotal = float(line.split('=')[-1])*Hartree  # second occurrence!
            for line in iter(text.split('\n')):
                if line.rfind('>>>> etotal (dc)=') > -1:
                    efree = float(line.split('=')[-1])*Hartree
        else:
            for line in iter(text.split('\n')):
                if line.rfind('>>>>> internal e=') > -1:
                    etotal = float(line.split('=')[-1])*Hartree  # first occurrence!
                    break
            for line in iter(text.split('\n')):
                if line.rfind('>>>>>>>>> etotal=') > -1:
                    efree = float(line.split('=')[-1])*Hartree
        if efree is None:
            raise RuntimeError('Total energy not found')
        if etotal is None:
            etotal = efree

        # Energy extrapolated to zero Kelvin:
        self.results['energy'] = (etotal + efree) / 2
        self.results['free_energy'] = efree

        # Forces:
        for line in lines:
            if line.rfind('cartesian forces (ev/angstrom) at end:') > -1:
                forces = []
                for i in range(natoms):
                    forces.append(np.array(
                            [float(f) for f in next(lines).split()[1:]]))
                self.results['forces'] = np.array(forces)
                break
        else:
            raise RuntimeError
        #
        self.width = self.read_electronic_temperature()
        self.nband = self.read_number_of_bands()
        self.niter = self.read_number_of_iterations()
        self.nelect = self.read_number_of_electrons()
        self.results['magmom'] = self.read_magnetic_moment()


    def initialize(self, atoms, raise_exception=True):

        self.species = list(set(atoms.get_atomic_numbers()))
        self.spinpol = atoms.get_initial_magnetic_moments().any()
        self.ppp_list = self.get_ppp_list(self.species, atoms, raise_exception)


    def get_ppp_list(self, species, atoms, raise_exception):

        ppp_list = []

        pppaths = os.environ.get('ABINIT_PP_PATH','.').split(':')
        xcname = 'GGA' if self.parameters.xc != 'LDA' else 'LDA'
        pps = self.parameters.pps
        for Z in species:
            number = abs(Z)
            symbol = chemical_symbols[number]

            names  = []
            for s in [ symbol, symbol.lower() ]:
                for xcn in [ xcname, xcname.lower() ]:
                    if pps in ['paw']:
                        hghtemplate = '%s-%s-%s.paw'  # E.g. "H-GGA-hard-uspp.paw"
                        names.append(hghtemplate % (s, xcn, '*'))
                        names.append('%s[.-_]*.paw'   % s)
                    elif pps in ['pawxml']:
                        hghtemplate = '%s.%s%s.xml'  # E.g. "H.GGA_PBE-JTH.xml"
                        names.append(hghtemplate % (s, xcn, '*'))
                        names.append('%s[.-_]*.xml'   % s)
                    elif pps in ['hgh.k']:
                        hghtemplate = '%s-q%s.hgh.k'  # E.g. "Co-q17.hgh.k"
                        names.append(hghtemplate % (s, '*'))
                        names.append('%s[.-_]*.hgh.k' % s)
                        names.append('%s[.-_]*.hgh' % s)
                    elif pps in ['tm']:
                        hghtemplate = '%d%s%s.pspnc'  # E.g. "44ru.pspnc"
                        names.append(hghtemplate % (number, s, '*'))
                        names.append('%s[.-_]*.pspnc' % s)
                    elif pps in ['hgh', 'hgh.sc']:
                        hghtemplate = '%d%s.%s.hgh'  # E.g. "42mo.6.hgh"
                        # There might be multiple files with different valence
                        # electron counts, so we must choose between
                        # the ordinary and the semicore versions for some elements.
                        #
                        # Therefore we first use glob to get all relevant files,
                        # then pick the correct one afterwards.
                        names.append(hghtemplate % (number, s, '*'))
                        names.append('%d%s%s.hgh' % (number, s, '*'))
                        names.append('%s[.-_]*.hgh' % s)
                    else: # default extension
                        names.append('%02d-%s.%s.%s' % (number, s, xcn, pps))
                        names.append('%02d[.-_]%s*.%s'   % (number, s, pps))
                        names.append('%02d%s*.%s'   % (number, s, pps))
                        names.append('%s[.-_]*.%s'   % (s, pps))

            found = False
            for name in names:        # search for file names possibilities
                for path in pppaths:  # in all available directories
                    filenames = glob(join(path, name))
                    if not filenames:
                        continue
                    if pps == 'paw':
                        # warning: see download.sh in
                        # abinit-pseudopotentials*tar.gz for additional
                        # information!
                        filenames[0] = max(filenames)  # Semicore or hard
                    elif pps == 'hgh':
                        filenames[0] = min(filenames)  # Lowest valence electron count
                    elif pps == 'hgh.k':
                        filenames[0] = max(filenames)  # Semicore - highest electron count
                    elif pps == 'tm':
                        filenames[0] = max(filenames)  # Semicore - highest electron count
                    elif pps == 'hgh.sc':
                        filenames[0] = max(filenames)  # Semicore - highest electron count

                    if filenames:
                        found = True
                        ppp_list.append(filenames[0])
                        break
                if found:
                    break

            if not found:
                ppp_list.append("Provide {}.{}.{}?".format(symbol, '*', pps))
                if raise_exception:
                    raise RuntimeError('Could not find {} pseudopotential {} for {}'.format(xcname.lower(), pps, symbol))

        return ppp_list


    def get_number_of_iterations(self):
        return self.niter

    def read_number_of_iterations(self):
        niter = None
        for line in open(self.label + '.txt'):
            if line.find(' At SCF step') != -1: # find the last iteration number
                niter = int(line.split()[3].rstrip(','))
        return niter

    def get_electronic_temperature(self):
        return self.width * Hartree

    def read_electronic_temperature(self):
        width = None
        # only in log file!
        for line in open(self.label + '.log'):  # find last one
            if line.find('tsmear') != -1:
                width = float(line.split()[1].strip())
        return width

    def get_number_of_electrons(self):
        return self.nelect

    def read_number_of_electrons(self):
        nelect = None
        # only in log file!
        for line in open(self.label + '.log'):  # find last one
            if line.find('with nelect') != -1:
                nelect = float(line.split('=')[1].strip())
        return nelect

    def get_number_of_bands(self):
        return self.nband

    def read_number_of_bands(self):
        nband = None
        for line in open(self.label + '.txt'): # find last one
            if line.find('     nband') != -1: # nband, or nband1, nband*
                nband = int(line.split()[-1].strip())
        return nband

    def get_kpts_info(self, kpt=0, spin=0, mode='eigenvalues'):
        return self.read_kpts_info(kpt, spin, mode)

    def get_k_point_weights(self):
        return self.get_kpts_info(kpt=0, spin=0, mode='k_point_weights')

    def get_bz_k_points(self):
        raise NotImplementedError

    def get_ibz_k_points(self):
        return self.get_kpts_info(kpt=0, spin=0, mode='ibz_k_points')

    def get_spin_polarized(self):
        return self.spinpol

    def get_number_of_spins(self):
        return 1 + int(self.spinpol)

    def read_magnetic_moment(self):
        magmom = None
        if not self.get_spin_polarized():
            magmom = 0.0
        else: # only for spinpolarized system Magnetisation is printed
            for line in open(self.label + '.txt'):
                if line.find('Magnetisation') != -1: # last one
                    magmom = float(line.split('=')[-1].strip())
        return magmom

    def get_fermi_level(self):
        return self.read_fermi()

    def get_eigenvalues(self, kpt=0, spin=0):
        return self.get_kpts_info(kpt, spin, 'eigenvalues')

    def get_occupations(self, kpt=0, spin=0):
        return self.get_kpts_info(kpt, spin, 'occupations')

    def read_fermi(self):
        """Method that reads Fermi energy in Hartree from the output file
        and returns it in eV"""
        E_f=None
        filename = self.label + '.txt'
        text = open(filename).read().lower()
        assert 'error' not in text
        for line in iter(text.split('\n')):
            if line.rfind('fermi (or homo) energy (hartree) =') > -1:
                E_f = float(line.split('=')[1].strip().split()[0])
        return E_f*Hartree

    def read_kpts_info(self, kpt=0, spin=0, mode='eigenvalues'):
        """ Returns list of last eigenvalues, occupations, kpts weights, or
        kpts coordinates for given kpt and spin.
        Due to the way of reading output the spins are exchanged in spin-polarized case.  """
        # output may look like this (or without occupation entries); 8 entries per line:
        #
        #  Eigenvalues (hartree) for nkpt=  20  k points:
        # kpt#   1, nband=  3, wtk=  0.01563, kpt=  0.0625  0.0625  0.0625 (reduced coord)
        #  -0.09911   0.15393   0.15393
        #      occupation numbers for kpt#   1
        #   2.00000   0.00000   0.00000
        # kpt#   2, nband=  3, wtk=  0.04688, kpt=  0.1875  0.0625  0.0625 (reduced coord)
        # ...
        #
        assert mode in ['eigenvalues', 'occupations', 'ibz_k_points',
                        'k_point_weights'], mode
        if self.get_spin_polarized():
            spin = {0: 1, 1: 0}[spin]
        if spin == 0:
            spinname = ''
        else:
            spinname = 'SPIN UP'.lower()
        # number of lines of eigenvalues/occupations for a kpt
        nband = self.get_number_of_bands()
        n_entries_float = 8  # float entries per line
        n_entry_lines = max(1, int((nband - 0.1) / n_entries_float) + 1)

        filename = self.label + '.txt'
        text = open(filename).read().lower()
        assert 'error' not in text
        lines = text.split('\n')
        text_list = []
        # find the beginning line of last eigenvalues
        contains_eigenvalues = 0
        for n, line in enumerate(lines):
            if spin == 0:
                if line.rfind('eigenvalues (hartree) for nkpt') > -1:
                #if line.rfind('eigenvalues (   ev  ) for nkpt') > -1: #MDTMP
                    contains_eigenvalues = n
            else:
                if (line.rfind('eigenvalues (hartree) for nkpt') > -1 and
                    line.rfind(spinname) > -1): # find the last 'SPIN UP'
                        contains_eigenvalues = n
        # find the end line of eigenvalues starting from contains_eigenvalues
        text_list = [lines[contains_eigenvalues]]
        for line in lines[contains_eigenvalues + 1:]:
            text_list.append(line)
            # find a blank line or eigenvalues of second spin
            if (not line.strip() or
                line.rfind('eigenvalues (hartree) for nkpt') > -1):
                break
        # remove last (blank) line
        text_list = text_list[:-1]

        assert contains_eigenvalues, 'No eigenvalues found in the output'

        n_kpts = int(text_list[0].split('nkpt=')[1].strip().split()[0])

        # get rid of the "eigenvalues line"
        text_list = text_list[1:]

        # join text eigenvalues description with eigenvalues
        # or occupation numbers for kpt# with occupations
        contains_occupations = False
        for line in text_list:
            if line.rfind('occupation numbers') > -1:
                contains_occupations = True
                break
        if mode == 'occupations':
            assert contains_occupations, 'No occupations found in the output'

        if contains_occupations:
            range_kpts = 2*n_kpts
        else:
            range_kpts = n_kpts

        values_list = []
        offset = 0
        for kpt_entry in range(range_kpts):
            full_line = ''
            for entry_line in range(n_entry_lines+1):
                full_line = full_line+str(text_list[offset+entry_line])
            first_line = text_list[offset]
            if mode == 'occupations':
                if first_line.rfind('occupation numbers') > -1:
                    # extract numbers
                    full_line = [float(v) for v in full_line.split('#')[1].strip().split()[1:]]
                    values_list.append(full_line)
            elif mode in ['eigenvalues', 'ibz_k_points', 'k_point_weights']:
                if first_line.rfind('reduced coord') > -1:
                    # extract numbers
                    if mode == 'eigenvalues':
                        full_line = [Hartree*float(v) for v in full_line.split(')')[1].strip().split()[:]]
                        #full_line = [float(v) for v in full_line.split(')')[1].strip().split()[:]] #MDTMP
                    elif mode == 'ibz_k_points':
                        full_line = [float(v) for v in full_line.split('kpt=')[1].strip().split('(')[0].split()]
                    else:
                        full_line = float(full_line.split('wtk=')[1].strip().split(',')[0].split()[0])
                    values_list.append(full_line)
            offset = offset+n_entry_lines+1

        if mode in ['occupations', 'eigenvalues']:
            return np.array(values_list[kpt])
        else:
            return np.array(values_list)



import numpy as np
import matplotlib.pyplot as plt
from ase.dft import Wannier
from gpaw import restart

atoms, calc = restart('benzene.gpw', txt=None)
wan = Wannier(nwannier=18, calc=calc, fixedstates=15, file='wan18.pickle')

weight_n = np.sum(abs(wan.V_knw[0])**2, 1)
N = len(weight_n)
F = wan.fixedstates_k[0]
plt.figure(1, figsize=(12, 4))
plt.bar(range(1, N + 1), weight_n, width=0.65, bottom=0,
        color='k', edgecolor='k', linewidth=None,
        align='center', orientation='vertical')
plt.plot([F + 0.5, F + 0.5], [0, 1], 'k--')
plt.axis(xmin=0.32, xmax=N + 1.33, ymin=0, ymax=1)
plt.xlabel('Eigenstate')
plt.ylabel('Projection of wannier functions')
plt.savefig('spectral_weight.png')
plt.show()



# creates: formatoptions.rst
from ase.io.formats import all_formats
with open('formatoptions.rst', 'w') as fd:
    print('============================================', file=fd)
    print('Format Specific Options', file=fd)
    print('============================================', file=fd)
    format_names = list(sorted(all_formats.keys()))
    for name in format_names:
        fmt = all_formats[name]
        print(".. _{:}:\n".format(name), file=fd)
        print(name, file=fd)
        print('----------------------------------------', file=fd)
        if fmt.can_read:
            print('\n .. autofunction:: {:}.read_{:}\n\n'.format(fmt.module_name,fmt._formatname), file=fd)
        if fmt.can_write:
            print('\n .. autofunction:: {:}.write_{:}\n\n'.format(fmt.module_name,fmt._formatname), file=fd)
        if (not fmt.can_read) and (not fmt.can_write):
            print('\n No automatic documentation of this module found.', file=fd)
            print('\n Visit https://gitlab.com/ase/ase/tree/master/ase/io/{:}.py to see if you find the information needed in the source code.\n\n'.format(fmt.module_name), file=fd)




import numpy as np

import ase.units as units
from ase.calculators.calculator import Calculator, all_changes
from ase.data import atomic_masses
from ase.geometry import find_mic

# Electrostatic constant
k_c = units.Hartree * units.Bohr

# Force field parameters
q_me = 0.206
q_c = 0.247
q_n = -0.453
sigma_me = 3.775
sigma_c = 3.650
sigma_n = 3.200
epsilon_me = 0.7824 * units.kJ / units.mol
epsilon_c = 0.544 * units.kJ / units.mol
epsilon_n = 0.6276 * units.kJ / units.mol
r_mec = 1.458
r_cn = 1.157
r_men = r_mec + r_cn
m_me = atomic_masses[6] + 3 * atomic_masses[1]
m_c = atomic_masses[6]
m_n = atomic_masses[7]


def combine_lj_lorenz_berthelot(sigma, epsilon):
    """Combine LJ parameters according to the Lorenz-Berthelot rule"""
    sigma_c = np.zeros((len(sigma), len(sigma)))
    epsilon_c = np.zeros_like(sigma_c)

    for ii in range(len(sigma)):
        sigma_c[:, ii] = (sigma[ii] + sigma) / 2
        epsilon_c[:, ii] = (epsilon[ii] * epsilon) ** 0.5
    return sigma_c, epsilon_c


class ACN(Calculator):
    implemented_properties = ['energy', 'forces']
    nolabel = True

    def __init__(self, rc=5.0, width=1.0):
        """Three-site potential for acetonitrile.

        Atom sequence must be:
        MeCNMeCN ... MeCN or NCMeNCMe ... NCMe

        When performing molecular dynamics (MD), forces are redistributed
        and only Me and N sites propagated based on a scheme for MD of
        rigid triatomic molecules from Ciccotti et al. Molecular Physics
        1982 (http://dx.doi.org/10.1080/00268978200100942). Apply constraints
        using the FixLinearTriatomic to fix the geometry of the acetonitrile
        molecules.

        rc: float
            Cutoff radius for Coulomb interactions.
        width: float
            Width for cutoff function for Coulomb interactions.

        References:

        http://dx.doi.org/10.1080/08927020108024509
        """
        self.rc = rc
        self.width = width
        self.forces = None
        Calculator.__init__(self)
        self.sites_per_mol = 3
        self.pcpot = None

    def calculate(self, atoms=None,
                  properties=['energy'],
                  system_changes=all_changes):
        Calculator.calculate(self, atoms, properties, system_changes)

        Z = atoms.numbers
        masses = atoms.get_masses()
        if Z[0] == 7:
            n = 0
            me = 2
            sigma = np.array([sigma_n, sigma_c, sigma_me])
            epsilon = np.array([epsilon_n, epsilon_c, epsilon_me])
        else:
            n = 2
            me = 0
            sigma = np.array([sigma_me, sigma_c, sigma_n])
            epsilon = np.array([epsilon_me, epsilon_c, epsilon_n])
        assert (Z[n::3] == 7).all(), 'incorrect atoms sequence'
        assert (Z[1::3] == 6).all(), 'incorrect atoms sequence'
        assert (masses[n::3] == m_n).all(), 'incorrect masses'
        assert (masses[1::3] == m_c).all(), 'incorrect masses'
        assert (masses[me::3] == m_me).all(), 'incorrect masses'

        R = self.atoms.positions.reshape((-1, 3, 3))
        pbc = self.atoms.pbc
        cd = self.atoms.cell.diagonal()
        nm = len(R)

        assert (self.atoms.cell == np.diag(cd)).all(), 'not orthorhombic'
        assert ((cd >= 2 * self.rc) | ~pbc).all(), 'cutoff too large'

        charges = self.get_virtual_charges(atoms[:3])

        # LJ parameters
        sigma_co, epsilon_co = combine_lj_lorenz_berthelot(sigma, epsilon)

        energy = 0.0
        self.forces = np.zeros((3 * nm, 3))

        for m in range(nm - 1):
            Dmm = R[m + 1:, 1] - R[m, 1]
            # MIC PBCs
            Dmm_min, Dmm_min_len = find_mic(Dmm, atoms.cell, pbc)
            shift = Dmm_min - Dmm

            # Smooth cutoff
            cut, dcut = self.cutoff(Dmm_min_len)

            for j in range(3):
                D = R[m + 1:] - R[m, j] + shift[:, np.newaxis]
                D_len2 = (D**2).sum(axis=2)
                D_len = D_len2**0.5
                # Coulomb interactions
                e = charges[j] * charges / D_len * k_c
                energy += np.dot(cut, e).sum()
                F = (e / D_len2 * cut[:, np.newaxis])[:, :, np.newaxis] * D
                Fmm = -(e.sum(1) * dcut / Dmm_min_len)[:, np.newaxis] * Dmm_min
                self.forces[(m + 1) * 3:] += F.reshape((-1, 3))
                self.forces[m * 3 + j] -= F.sum(axis=0).sum(axis=0)
                self.forces[(m + 1) * 3 + 1::3] += Fmm
                self.forces[m * 3 + 1] -= Fmm.sum(0)
                # LJ interactions
                c6 = (sigma_co[:, j]**2 / D_len2)**3
                c12 = c6**2
                e = 4 * epsilon_co[:, j] * (c12 - c6)
                energy += np.dot(cut, e).sum()
                F = (24 * epsilon_co[:, j] * (2 * c12 -
                     c6) / D_len2 * cut[:, np.newaxis])[:, :, np.newaxis] * D
                Fmm = -(e.sum(1) * dcut / Dmm_min_len)[:, np.newaxis] * Dmm_min
                self.forces[(m + 1) * 3:] += F.reshape((-1, 3))
                self.forces[m * 3 + j] -= F.sum(axis=0).sum(axis=0)
                self.forces[(m + 1) * 3 + 1::3] += Fmm
                self.forces[m * 3 + 1] -= Fmm.sum(0)

        if self.pcpot:
            e, f = self.pcpot.calculate(np.tile(charges, nm),
                                        self.atoms.positions)
            energy += e
            self.forces += f

        self.results['energy'] = energy
        self.results['forces'] = self.forces

    def redistribute_forces(self, forces):
        return forces

    def get_molcoms(self, nm):
        molcoms = np.zeros((nm, 3))
        for m in range(nm):
            molcoms[m] = self.atoms[m * 3:(m + 1) * 3].get_center_of_mass()
        return molcoms

    def cutoff(self, d):
        x1 = d > self.rc - self.width
        x2 = d < self.rc
        x12 = np.logical_and(x1, x2)
        y = (d[x12] - self.rc + self.width) / self.width
        cut = np.zeros(len(d))  # cutoff function
        cut[x2] = 1.0
        cut[x12] -= y**2 * (3.0 - 2.0 * y)
        dtdd = np.zeros(len(d))
        dtdd[x12] -= 6.0 / self.width * y * (1.0 - y)
        return cut, dtdd

    def embed(self, charges):
        """Embed atoms in point-charges."""
        self.pcpot = PointChargePotential(charges)
        return self.pcpot

    def check_state(self, atoms, tol=1e-15):
        system_changes = Calculator.check_state(self, atoms, tol)
        if self.pcpot and self.pcpot.mmpositions is not None:
            system_changes.append('positions')
        return system_changes

    def add_virtual_sites(self, positions):
        return positions  # no virtual sites

    def get_virtual_charges(self, atoms):
        charges = np.empty(len(atoms))
        Z = atoms.numbers
        if Z[0] == 7:
            n = 0
            me = 2
        else:
            n = 2
            me = 0
        charges[me::3] = q_me
        charges[1::3] = q_c
        charges[n::3] = q_n
        return charges


class PointChargePotential:
    def __init__(self, mmcharges):
        """Point-charge potential for ACN.

        Only used for testing QMMM.
        """
        self.mmcharges = mmcharges
        self.mmpositions = None
        self.mmforces = None

    def set_positions(self, mmpositions):
        self.mmpositions = mmpositions

    def calculate(self, qmcharges, qmpositions):
        energy = 0.0
        self.mmforces = np.zeros_like(self.mmpositions)
        qmforces = np.zeros_like(qmpositions)
        for C, R, F in zip(self.mmcharges, self.mmpositions, self.mmforces):
            d = qmpositions - R
            r2 = (d**2).sum(1)
            e = units.Hartree * units.Bohr * C * r2**-0.5 * qmcharges
            energy += e.sum()
            f = (e / r2)[:, np.newaxis] * d
            qmforces += f
            F -= f.sum(0)
        self.mmpositions = None
        return energy, qmforces

    def get_forces(self, calc):
        return self.mmforces



# flake8: noqa
""" class Wannier provides the function get_wannier_localization_matrix and initial_wannier.
"""

import numpy as np
from Scientific.IO.NetCDF import NetCDFFile as netCDF

def dagger(matrix,copy=1):
        # First change the axis: (Does not allocate a new array)
        matrix_conj=np.swapaxes(matrix,0,1)
        if copy: # Allocate space for new array
                return np.conjugate(matrix_conj)
        else:    # The array of matrix is used for output
                np.dot(matrix_conj.imag,-1,matrix_conj.imag)
                return matrix_conj

def project(a,b):
    """ returns the projection of b onto a"""
    return a*(np.dot(np.conjugate(a),b)/np.dot(np.conjugate(a),a))

def translate(array,translation):
        """Method for translating an array"""
        newarray=array
        size=array.shape
        for dim in range(len(translation)):
            axis=dim-len(translation)
            newarray=np.concatenate((np.take(newarray,list(range(translation[dim],size[axis])),axis),np.take(newarray,list(range(translation[dim])),axis)),axis)
        return newarray.copy()

def cartesian2scaled(basis,cart):
    return np.linalg.solve(np.transpose(basis),cart)

def scaled2cartesian(basis,scaled):
    return np.dot(scaled,basis)

class Translation_Operator:
    def __init__(self,dimensions,basis):
        self.set_dimensions(dimensions)
        self.set_basis(basis)

    def set_dimensions(self,dim):
        self.dimensions = dim

    def get_dimensions(self):
        return self.dimensions

    def set_basis(self,basis):
        self.basis = basis

    def get_basis(self):
        return self.basis

    def set_cartesian_translation_coordinates(self,trans_cartes):
        self.cartesian_translation_coordinates = trans_cartes

    def get_cartesian_translation_coordinates(self):
        return self.cartesian_translation_coordinates

    def get_coordinates(self):
        trans_cartes = self.get_cartesian_translation_coordinates()
        #trans_coor = np.linalg.solve(np.transpose(self.get_basis()),trans_cartes)
        trans_coor = cartesian2scaled(self.get_basis(),trans_cartes)
        return np.multiply(trans_coor,self.get_dimensions())

    def get_translational_diagonal(self,dim_index):
        # Using that G = 2*pi*n/L : epx(-i*G*x_{1}) =
        #                pow(exp(-i*2*pi/N*x_{1}),n)

        length = self.get_dimensions()[dim_index]
        coordinates = self.get_coordinates()[dim_index]
        basis_neg = np.arange(0, -length // 2, -1)
        basis_pos = np.arange(length // 2, 0, -1)
        basis = np.concatenate((basis_neg,basis_pos),-1)
        prefactor=np.exp(-complex(0,1)*2*np.pi*coordinates/length)
        translation=list(map(lambda x,prefactor=prefactor:prefactor**x,basis))
        return np.array(translation)

    def operate(self,state):
        translation = np.multiply.outer(self.get_translational_diagonal(0),self.get_translational_diagonal(1))
        translation = np.multiply.outer(translation,self.get_translational_diagonal(2))
        return np.multiply(translation,state)

def coordinate_array_from_unit_vectors(shape, gridunitvectors,
                                       origin=[0, 0, 0], indexfunction=None):
        """
        This method can be used to obtain an array representing the coordinates
        of a space defined by 'gridunitvecors'. 'gridunitvectors' is in turn a
        list containing the vectors defining the cells of the grid, i.e. the
        vectors between neighboring grid points. These vectors are spanned
        according to the specified shape.

        'origin' -- specifies the origin of the returned coordinate array.

        'indexfunction' -- is a lambda expression that defines the indices
        with which each of the specified gridunitvectors are to be multiplied.
        'indexfunction' must take two arguments, 'i' and 'length' - default
        is 'lambda i,length:i'. During execution the input index 'i' will run
        over the interval 0,1,..., 'length' -1.

        **An Example**

        To obtain a coordinate array of shape (10,10) with
        'gridunitvectors' =[[2,0],[0,1]] and the origin at [10,0] use:

        'CoordinateArrayFromUnitVectors((10,10),[[2,0],[0,1],[10,0])'

        Note that the output array will be of shape
        (< *dimension* > ,  < *spatialcoordinates* >).
        """

        if indexfunction is None:
            indexfunction = lambda i, length: i

        coordinatelist=[]
        gridunitvectors=np.asarray(gridunitvectors)
        # Looping over the dimensionality of the vectors
        for dim in range(gridunitvectors.shape[1]):
                coordinates=origin[dim]
                # Contribution from each unitvector
                for nunitvector in range(gridunitvectors.shape[0]):
                        # Finding the indices from which the coordinate grid
                        # is spanned
                        indices=list(map(lambda i,f=indexfunction,l=shape[nunitvector]:f(i,l),list(range(shape[nunitvector]))))
                        coordinatefunc=lambda i,v=gridunitvectors[nunitvector,dim]:i*v
                        coordinates=np.add.outer(coordinates,list(map(coordinatefunc,indices)))
                coordinatelist.append(coordinates)
        return np.array(coordinatelist)

def coordinates_from_function_values(dimensions,functionvalues):
        # In general: Using basis functions of the form:
        # 1/sqrt(N1*N2*N3)*exp(iG_dot_r)
        normalization=np.sqrt(np.multiply.reduce(dimensions))
        # Functionvalues of the form (a,x_i,x_j,x_k), where
        # a is different functions and x_i,x_j,x_k the function values
        # Return array of the form (a,G_i,G_j,G_k)

        # The three last axes are transformed
        coordinates=np.fft.ifft(np.fft.ifft(np.fft.ifft(functionvalues,n=functionvalues.shape[-1],axis=-1),n=functionvalues.shape[-2],axis=-2),n=functionvalues.shape[-3],axis=-3)
        # Scaling:
        np.multiply(coordinates,normalization,coordinates)
        return coordinates


class Wannier:

   def __init__(self,calc):

      self.calc = calc
      self.set_bands(None)
      self.set_spin(None)
      self.has_changed = True

   def set_spin(self,spin):
      self.spin = spin
      self.has_changed = True

   def get_spin(self):
      return self.spin

   def get_fft_index(self):
      if not hasattr(self,'fftindex'):
          fftindex = []
          kpoints = self.calc.get_bz_k_points()
          for kpt in range(len(kpoints)):
              fftindex.append(self.calc.get_reciprocal_fft_index())
          self.fftindex = fftindex
      return self.fftindex

   def get_grid_dimensions(self):
      fftgrids = self.calc.get_fftgrid()
      return fftgrids['soft']

   def get_list_of_wave_functions(self):
      if self.get_spin() == None or self.get_bands() == None:
          raise RuntimeError('Bands and spin must be set before wave ' +
                             'function list can be created')

      if self.has_changed:
          listofwavefct = []
          kpoints = self.calc.get_bz_k_points()
          for kpt in range(len(kpoints)):
              eigenstates = []
              for band in range(self.get_bands()):
                  eigenstates.append(self.calc.get_reciprocal_bloch_function(band=band,kpt=kpt,spin=self.get_spin()))

              listofwavefct.append(eigenstates)
          self.listofwavefct = listofwavefct
          self.has_changed = False
      return self.listofwavefct

   def set_bands(self,numberofbands):
      self.numberofbands = numberofbands
      self.has_changed = True

   def get_bands(self):
      return self.numberofbands

   def get_zi_bloch_matrix(self,dirG,kpoint,nextkpoint,G_I):
      """ calculate matrix of ZIi,j values
      This matrix consist of 3 matrices each of dimension MxM, i.e. corresponding to the full space.
      """

      #print 'DacapoWannier: Initialize ZIBlochMatrix ..'
      if self.get_bands() == None or self.get_spin() == None:
          raise RuntimeError('Bands and spin must be set before wannier '
                             'localization matrix can be calculated')

      phi=np.swapaxes(np.array(self.get_list_of_wave_functions()[kpoint]),0,1)
      # K1 and reciprocal lattice vector G_I  given kpoint K
      # that fulfills the criteria : K1-K-K0+G1=0
      list1,list2 = self.get_gg_list(kpoint,nextkpoint,G_I)

      a=np.take(phi,list1,axis=0)
      a=np.swapaxes(a,0,1)
      phi1 = np.swapaxes(np.array(self.get_list_of_wave_functions()[nextkpoint]),0,1)
      b=np.take(phi1,list2,axis=0)

      ziblochmatrix = np.dot(np.conjugate(a),b)
      usziblochmatrix = self.get_ultra_soft_non_loc_matrix(dirG,kpoint,nextkpoint)
      ziblochmatrix += usziblochmatrix

      return ziblochmatrix


   def get_gg_list(self,kpt1,kpt2,GI):
       """ define list of (G,G+G1) defining the product
       phi(kpt1,G)*phi(kpt2,G+G1),

       GI is one of
       [[1,0,0],[0,1,0],[0,0,1],[1,1,0],[1,0,1],[0,1,1]]

       The layout of fourier components is
       1   2   3   4   5   6   7   8   ngx = 8
       0   1   2   3   4  -3  -2  -1    n*2pi/L
       """

       numberplanewaves = len(self.get_list_of_wave_functions()[kpt1][0])
       reciprocalindex = self.get_fft_index()[kpt1]

       ngrids = self.get_grid_dimensions()

       # setup the mapping from the 3D FFT grid to the wavefuction list
       map2 = self.get_index_map(kpt2)

       # gglist = []
       # print "Generating plane wave index list for direction ",GI," kpt12 ",kpt1,kpt2
       list1 = []
       list2 = []

       # find G,G+GI
       for n in range(numberplanewaves):
           index = reciprocalindex[:,n]
           index = index - 1
           # find G,G+GI
           for dir in range(3):
               index[dir] += GI[dir]
               if index[dir]>=ngrids[dir]:
                   # wrap around
                   index[dir] = 0

           # now find the corresponding index into phi(kpt2)
           n1 = map2[index[0],index[1],index[2]]

           if n1>=0:
              list1.append(n)
              list2.append(n1)

       # print '  Number of elements in GG list ',len(list1)
       return list1,list2



   def get_index_map(self,kpt):
       """ generate mapping from 3D FFT grid to the wavefunction list

       A negative number is returned from map(g1,g2,g3) is the
       grid point does not exists in the wavefunction list
       """
       ngrids = self.get_grid_dimensions()
       map_to_wflist = np.zeros(ngrids,np.int)
       map_to_wflist = map_to_wflist - 1

       numberplanewaves = len(self.get_list_of_wave_functions()[kpt][0])
       reciprocalindex = self.get_fft_index()[kpt]

       for n in range(numberplanewaves):
           i0 = reciprocalindex[0][n]-1
           i1 = reciprocalindex[1][n]-1
           i2 = reciprocalindex[2][n]-1
           map_to_wflist[i0,i1,i2] = n

       return map_to_wflist

   def get_ultra_soft_non_loc_matrix(self,GI,kpt,kpt1):
        """ calculate
              a                            I        I              I
             W    = sum(n,m,I) <psi  | beta  > <beta  | psi   > * q
               i,j                  ik      m        n      jk1     mn

             n,m : projectors
             I   : atom no
             a (nbands,nbands) matrix is returned.
        """

        nc = netCDF(self.calc.get_nc(),'r')
        if not 'WannierAugFactor' in nc.variables:
            nc.close()
            return

        # Read NLProjectorPsi, StructureFactor, WannierAugFactor
        vnlprojpsi = nc.variables['NLProjectorPsi'][:]
        nlprojpsi = np.zeros((np.shape(vnlprojpsi[:,:,:,:,:,0])),np.complex)
        nlprojpsi.real = vnlprojpsi[:,:,:,:,:,0]
        nlprojpsi.imag = vnlprojpsi[:,:,:,:,:,1]
        nlprojpsi = np.swapaxes(nlprojpsi,3,4)

        vstrfactor = nc.variables['StructureFactor'][:]
        strfactor = np.zeros((np.shape(vstrfactor[:,:,0])),np.complex)
        strfactor.real = vstrfactor[:,:,0]
        strfactor.imag = vstrfactor[:,:,1]

        vaugfactor = nc.variables['WannierAugFactor'][:]
        augfactor = np.zeros((np.shape(vaugfactor[:,:,:,:,0])),np.complex)
        augfactor.real = vaugfactor[:,:,:,:,0]
        augfactor.imag = vaugfactor[:,:,:,:,1]
        augfactor = np.swapaxes(augfactor,0,1)

        nc.close()

        lst=[[1,0,0],[0,1,0],[0,0,1],[1,1,0],[1,0,1],[0,1,1]]

        # find direction corresponding to GI
        dir = lst.index(GI.tolist())

        natoms = nlprojpsi.shape[2]
        nbands = self.get_bands()
        matrix = np.zeros([nbands,nbands],np.complex)
        for atom in range(natoms):

            if kpt==kpt1:
                q=np.conjugate(augfactor[:,:,atom,dir+1])
            else:
                q=augfactor[:,:,atom,0]

                if dir<3:
                    q=q*np.conjugate(strfactor[atom,dir])

            A=nlprojpsi[kpt,self.get_spin(),atom,:nbands,:]
            B=nlprojpsi[kpt1,self.get_spin(),atom,:nbands,:]
            matrix=matrix+np.dot(A,np.dot(q,dagger(B)))
        return matrix

##### code to calculate initial wannier functions ######

   def set_data(self,data):
        self.data = data

   def set_k_point_grid(self,kpointgrid):
        self.kpointgrid = np.array(kpointgrid)

   def get_k_point_grid(self):
        return self.kpointgrid

   def get_repeated_unit_cell(self):
        basis=self.calc.get_atoms().get_cell()
        return np.transpose(np.transpose(basis)*self.get_k_point_grid())

   def get_repeated_grid_dimensions(self):
        return self.get_grid_dimensions()*self.get_k_point_grid()

   def get_detailed_data(self):
        if not hasattr(self,'detaileddata'):
            """ Coverts data on the form
                [atom,l,(m),a], or [center,l,(m),a].
                atom is a number in a bracket, example [2] for atom number 2
                center is 3 numbers in a bracket, example [1,0.5,0.3] denoting SCALED coordinates of the center.
                Here m is optional. If m is left out all m values are used."""
            datalist=self.data
            detaileddata=[]
            atoms=self.calc.get_atoms()
            for data in datalist:
                if len(data[0])==1:
                    r_c=atoms[data[0][0]].get_position()
                elif len(data[0])==3:
                    r_c = scaled2cartesian(atoms.get_cell(),data[0])
                else:
                    print("First element in initial data must be of the form [atom] or [c1,c2,c3], where the latter is scaled coordinates of the center")
                if len(data)==4:
                    # m is specified
                    detaileddata.append([r_c,data[1],data[2],data[3]])
                else:
                    # Orbitals with all allowed m values are produced
                    for m in range(-data[1],data[1]+1):
                        detaileddata.append([r_c,data[1],m,data[2]])
            self.detaileddata = detaileddata

        return self.detaileddata

   def get_origin_index(self):

        griddim = self.get_repeated_grid_dimensions()
        originindex = [int(coor) for coor in list(np.multiply(np.array([0.5,0.5,0.5]),griddim))]
        return originindex

   def get_cartesian_coordinates(self):

        griddim = self.get_repeated_grid_dimensions()
        basis = self.calc.get_atoms().get_cell()

        # Set origin to center of grid
        originindex = self.get_origin_index()
        # origincoord is in scaled coordinates:
        origincoord=-np.array(np.array(originindex,np.float)/griddim)
        origincart = scaled2cartesian(basis,origincoord)
        gridunitvectors = np.array(list(map(lambda unitvector,shape:unitvector/shape,basis,griddim)))
        c = coordinate_array_from_unit_vectors(shape=griddim,
                                           gridunitvectors=gridunitvectors,
                                           origin=origincart)
        return c

   def get_normalized_coordinates(self):

        if not hasattr(self,'normalized_coordinates'):
            originindex = tuple(self.get_origin_index())
            c = self.get_cartesian_coordinates()
            dist = np.sqrt(c[0]**2+c[1]**2+c[2]**2)

            # We define "normalized" coordinates. To avoid undeterminancy at origin we move
            # to the point (1,1,1)*1e-8
            c[0][originindex]=1.0e-8
            c[1][originindex]=1.0e-8
            c[2][originindex]=1.0e-8

            dist[originindex]=np.sqrt(3)*1.0e-8
            self.normalized_coordinates = c/dist

        return self.normalized_coordinates

   def get_distance_array_at_origin(self):

        # Set origin to center of grid
        originindex = self.get_origin_index()

        c = self.get_cartesian_coordinates()
        dist=np.sqrt(c[0]**2+c[1]**2+c[2]**2)
        # Translate back to origin
        dist=translate(dist,originindex)
        return dist

   def setup_m_matrix(self,listofeigenstates,bzkpoints):

        if self.data == None or self.kpointgrid == None:
            raise RuntimeError(
                'Must set data, kpointgrid, spin before calculating M matrix')

        fftindex = self.get_fft_index()

        unitcell=self.get_repeated_unit_cell()
        griddim = self.get_repeated_grid_dimensions()
        data = self.get_detailed_data()
        nkpoints = len(bzkpoints)
        nbands = len(listofeigenstates[0])
        M = np.zeros([nkpoints,nbands,len(data)],np.complex)
        orbital = np.zeros(griddim,np.complex)
        dist = self.get_distance_array_at_origin()

        transop = Translation_Operator(griddim,unitcell)
        rec_basis = 2*np.pi*np.linalg.inv(np.transpose(self.calc.get_atoms().get_cell()))
        large_rec_basis = 2*np.pi*np.linalg.inv(np.transpose(unitcell))

        for i in range(len(data)):
            # Translate orbital
            r_c=data[i][0]
            l,m=data[i][1],data[i][2]
            a=data[i][3]
            orbital = self.get_cubic_harmonic_at_origin(l,m)*np.exp(-dist/a)
            orbital_fft = coordinates_from_function_values(griddim,orbital)
            transop.set_cartesian_translation_coordinates(r_c)
            orbital_fft = transop.operate(orbital_fft)
            for kpt in range(nkpoints):
                kpoint = bzkpoints[kpt]
                kptnumber = cartesian2scaled(large_rec_basis,scaled2cartesian(rec_basis,kpoint))
                kptnumber[0]=round(kptnumber[0])
                kptnumber[1]=round(kptnumber[1])
                kptnumber[2]=round(kptnumber[2])
                kptnumber=kptnumber.astype(int)
                u_k = self.extract_periodic_part_of_small_cell(orbital_fft,kptnumber)
                compact_u_k = self.get_compact_fft_representation(u_k,fftindex[kpt],len(listofeigenstates[kpt][0]))
                M[kpt,:,i] = np.dot(np.conjugate(np.array(listofeigenstates[kpt])),compact_u_k)

        self.mmatrix = M

   def get_cubic_harmonic_at_origin(self,l,m):
       """ l=0,1,2. m=-l,...,l"""

       griddim = self.get_repeated_grid_dimensions()
       harmonic = np.zeros(griddim,np.complex)
       originindex = self.get_origin_index()
       nc = self.get_normalized_coordinates()

       # Constructing cubic harmonic
       if l==0 and m==0:
           harmonic=(1/np.sqrt(4*np.pi))*np.ones(nc[0].shape,np.Complex)
           harmonic=translate(harmonic,originindex)
       if l==1 and m==0:
           # p_x
           harmonic=np.sqrt(3/(4*np.pi))*nc[0]
           harmonic=translate(harmonic,originindex)
       if l==1 and m==-1:
           # p_z
           harmonic=np.sqrt(3/(4*np.pi))*nc[2]
           harmonic=translate(harmonic,originindex)
       if l==1 and m==1:
           # p_y
           harmonic=np.sqrt(3/(4*np.pi))*nc[1]
           harmonic=translate(harmonic,originindex)
       if l==2 and m==0:
           harmonic=0.5*np.sqrt(5/(4*np.pi))*(3*(nc[0]**2)-np.ones(nc[0].shape,np.Complex))
           harmonic=translate(harmonic,originindex)
       if l==2 and m==-1:
           harmonic=np.sqrt(15/(16*np.pi))*(nc[2]**2-nc[1]**2)
           harmonic=translate(harmonic,originindex)
       if l==2 and m==1:
           harmonic=np.sqrt(15/(4*np.pi))*nc[0]*nc[1]
           harmonic=translate(harmonic,originindex)
       if l==2 and m==-2:
           harmonic=np.sqrt(15/(4*np.pi))*nc[2]*nc[1]
           harmonic=translate(harmonic,originindex)
       if l==2 and m==2:
           harmonic=np.sqrt(15/(4*np.pi))*nc[2]*nc[0]
           harmonic=translate(harmonic,originindex)
       return harmonic

   def extract_periodic_part_of_small_cell(self,f,k):
       n1,n2,n3=self.get_k_point_grid()
       trans=[0,0,0]
       if k[0]<0:
           k[0]+=n1
           trans[0]=self.get_grid_dimensions()[0]-1
       if k[1]<0:
           k[1]+=n2
           trans[1]=self.get_grid_dimensions()[1]-1
       if k[2]<0:
           k[2]+=n3
           trans[2]=self.get_grid_dimensions()[2]-1

       u=f[k[0]::n1,k[1]::n2,k[2]::n3].copy()
       return translate(u,trans)

   def get_compact_fft_representation(self,freciprocal,fftindex,numberofpws):
       wflist=np.zeros([numberofpws],np.complex)
       for i in range(numberofpws):
           wflist[i]=freciprocal[int(fftindex[0,i]-1),int(fftindex[1,i]-1),int(fftindex[2,i]-1)]
       return wflist

   def get_orthonormality_factor(self,matrix):
       defect = abs(np.dot(dagger(matrix),matrix))-np.identity(matrix.shape[1],np.Float)
       return max(abs(defect.flat))

   def get_list_of_coefficients_and_rotation_matrices(self,matrixdimensions):
       from ase.dft.wannier import normalize,gram_schmidt
       import random
       M,N,L=matrixdimensions
       nkpt=len(N)
       Ulist=[]
       clist=[]
       if not hasattr(self,'mmatrix'):
           raise RuntimeError('Must setup M Matrix first!')
       coeffmatrix=self.mmatrix
       for kpt in range(nkpt):
           #First normalize the columns of coeffmatrix
           normalize(coeffmatrix[kpt])
           T=coeffmatrix[kpt][N[kpt]:].copy()
           numberoforbitals=T.shape[1]
           c=np.zeros([M-N[kpt],L[kpt]],np.complex)
           U=np.zeros([N[kpt]+L[kpt],N[kpt]+L[kpt]],np.complex)
           # Initialize weights
           w=abs(np.sum(T*np.conjugate(T)))
           for i in range(min(L[kpt],numberoforbitals)):
               # Find index of maximal element in w
               t=w.tolist().index(max(w))
               c[:,i]=T[:,t]
               # Orthogonalize c[:,i] on previous vectors
               for j in range(i):
                   c[:,i]=c[:,i]-project(c[:,j],T[:,t])
               c[:,i]=c[:,i]/np.sqrt(np.dot(c[:,i],np.conjugate(c[:,i])))
               # Update weights
               w=w-abs(np.dot(np.conjugate(c[:,i]),T))**2
           if numberoforbitals<L[kpt]:
               # Supplement c by random vectors
               for i in range(numberoforbitals,L[kpt]):
                   for j in range(M-N[kpt]):
                       c[j,i]=random.random()
               gram_schmidt(c)
           # Test whether columns are orthonormal
           if L[kpt]>0:
               test = self.get_orthonormality_factor(c)
               if test>1.0e-3:
                   print("ERROR: Columns of c not orthogonal!")

           U[:N[kpt],:numberoforbitals]=coeffmatrix[kpt][:N[kpt]]
           U[N[kpt]:,:numberoforbitals]=np.dot(dagger(c),coeffmatrix[kpt][N[kpt]:])
           # Perform democratic Lowdin orthogonalization on U[:,numberoforbitals]
           gram_schmidt(U[:,:numberoforbitals])
           if numberoforbitals<(N[kpt]+L[kpt]):
               #Supplement U by random vectors
               for i in range(numberoforbitals,N[kpt]+L[kpt]):
                   for j in range(N[kpt]+L[kpt]):
                       U[j,i]=random.random()
               # Finally orthogonalize everything
               # Note, only random vectors are affected
               gram_schmidt(U)
           # Test whether columns are orthonormal
           test = self.get_orthonormality_factor(U)
           if test>1.0e-3:
               print("ERROR: Columns of U not orthogonal for kpoint",kpt)
           Ulist.append(U)
           clist.append(c)

       return clist,Ulist




from ase.build import fcc100, add_adsorbate
from ase.constraints import FixAtoms
from ase.calculators.emt import EMT
from ase.dimer import DimerControl, MinModeAtoms, MinModeTranslate

# Set up a small "slab" with an adatoms
atoms = fcc100('Pt', size = (2, 2, 1), vacuum = 10.0)
add_adsorbate(atoms, 'Pt', 1.611, 'hollow')

# Freeze the "slab"
mask = [atom.tag > 0 for atom in atoms]
atoms.set_constraint(FixAtoms(mask = mask))

# Calculate using EMT
atoms.set_calculator(EMT())
relaxed_energy = atoms.get_potential_energy()

# Set up the dimer
d_control = DimerControl(initial_eigenmode_method = 'displacement', \
                         displacement_method = 'vector', logfile = None, \
                         mask = [0, 0, 0, 0, 1])
d_atoms = MinModeAtoms(atoms, d_control)

# Displace the atoms
displacement_vector = [[0.0]*3]*5
displacement_vector[-1][1] = -0.1
d_atoms.displace(displacement_vector = displacement_vector)

# Converge to a saddle point
dim_rlx = MinModeTranslate(d_atoms, trajectory = 'dimer_method.traj', \
                           logfile = None)
dim_rlx.run(fmax = 0.001)

# Test the results
tolerance = 1e-3
assert(d_atoms.get_barrier_energy() - 1.03733136918 < tolerance)
assert(abs(d_atoms.get_curvature() + 0.900467048707) < tolerance)
assert(d_atoms.get_eigenmode()[-1][1] < -0.99)
assert(abs(d_atoms.get_positions()[-1][1]) < tolerance)



from optparse import OptionParser

import numpy as np

from ase.atoms import Atoms
from ase.symbols import string2symbols
from ase.build import (molecule, fcc111, hcp0001, bcc110, bcc100, diamond111,
                       add_adsorbate)
from ase.data import reference_states, atomic_numbers, covalent_radii
from ase.io import write
from ase.visualize import view


def build():
    p = OptionParser(usage='%prog  [options] [ads@]surf [output file]',
                     version='%prog 0.1',
                     description='Example ads/surf: fcc-CO@2x2Ru0001')
    p.add_option('-l', '--layers', type='int',
                 default=4,
                 help='Number of layers.')
    p.add_option('-v', '--vacuum', type='float',
                 default=5.0,
                 help='Vacuum.')
    p.add_option('-x', '--crystal-structure',
                 help='Crystal structure.',
                 choices=['sc', 'fcc', 'bcc', 'hcp'])
    p.add_option('-a', '--lattice-constant', type='float',
                 help='Lattice constant in Angstrom.')
    p.add_option('--c-over-a', type='float',
                 help='c/a ratio.')
    p.add_option('--height', type='float',
                 help='Height of adsorbate over surface.')
    p.add_option('--distance', type='float',
                 help='Distance between adsorbate and nearest surface atoms.')
    p.add_option('-M', '--magnetic-moment', type='float', default=0.0,
                 help='Magnetic moment.')
    p.add_option('-G', '--gui', action='store_true',
                 help="Pop up ASE's GUI.")
    p.add_option('-P', '--python', action='store_true',
                 help="Write Python script.")

    opt, args = p.parse_args()

    if not 1 <= len(args) <= 2:
        p.error("incorrect number of arguments")

    if '@' in args[0]:
        ads, surf = args[0].split('@')
    else:
        ads = None
        surf = args[0]

    if surf[0].isdigit():
        i1 = surf.index('x')
        n = int(surf[:i1])
        i2 = i1 + 1
        while surf[i2].isdigit():
            i2 += 1
        m = int(surf[i1 + 1:i2])
        surf = surf[i2:]
    else:
        n = 1
        m = 1

    if surf[-1].isdigit():
        if surf[1].isdigit():
            face = surf[1:]
            surf = surf[0]
        else:
            face = surf[2:]
            surf = surf[:2]
    else:
        face = None

    Z = atomic_numbers[surf]
    state = reference_states[Z]

    if opt.crystal_structure:
        x = opt.crystal_structure
    else:
        x = state['symmetry']
    
    if opt.lattice_constant:
        a = opt.lattice_constant
    else:
        a = estimate_lattice_constant(surf, x, opt.c_over_a)

    script = ['from ase.build import ',
              'vac = %r' % opt.vacuum,
              'a = %r' % a]
    
    if x == 'fcc':
        if face is None:
            face = '111'
        slab = fcc111(surf, (n, m, opt.layers), a, opt.vacuum)
        script[0] += 'fcc111'
        script += ['slab = fcc111(%r, (%d, %d, %d), a, vac)' %
                   (surf, n, m, opt.layers)]
        r = a / np.sqrt(2) / 2
    elif x == 'bcc':
        if face is None:
            face = '110'
        if face == '110':
            slab = bcc110(surf, (n, m, opt.layers), a, opt.vacuum)
        elif face == '100':
            slab = bcc100(surf, (n, m, opt.layers), a, opt.vacuum)
        script[0] += 'bcc' + face
        script += ['slab = bcc%s(%r, (%d, %d, %d), a, vac)' %
                   (face, surf, n, m, opt.layers)]
        r = a * np.sqrt(3) / 4
    elif x == 'hcp':
        if face is None:
            face = '0001'
        if opt.c_over_a is None:
            c = np.sqrt(8 / 3.0) * a
        else:
            c = opt.c_over_a * a
        slab = hcp0001(surf, (n, m, opt.layers), a, c, opt.vacuum)
        script[0] += 'hcp0001'
        script += ['c = %r * a' % (c / a),
                   'slab = hcp0001(%r, (%d, %d, %d), a, c, vac)' %
                   (surf, n, m, opt.layers)]
        r = a / 2
    elif x == 'diamond':
        if face is None:
            face = '111'
        slab = diamond111(surf, (n, m, opt.layers), a, opt.vacuum)
        script[0] += 'diamond111'
        script += ['slab = diamond111(%r, (%d, %d, %d), a, vac)' %
                   (surf, n, m, opt.layers)]
        r = a * np.sqrt(3) / 8
    else:
        raise NotImplementedError

    magmom = opt.magnetic_moment
    if magmom is None:
        magmom = {'Ni': 0.6, 'Co': 1.2, 'Fe': 2.3}.get(surf, 0.0)
    slab.set_initial_magnetic_moments([magmom] * len(slab))
    if magmom != 0:
        script += ['slab.set_initial_magnetic_moments([%r] * len(slab))' %
                   magmom]
    
    slab.pbc = 1
    script += ['slab.pbc = True']
    
    name = '%dx%d%s%s' % (n, m, surf, face)

    if ads:
        site = 'ontop'
        if '-' in ads:
            site, ads = ads.split('-')

        name = site + '-' + ads + '@' + name
        symbols = string2symbols(ads)
        nads = len(symbols)
        if nads == 1:
            script[:0] = ['from ase import Atoms']
            script += ['ads = Atoms(%r)' % ads]
            ads = Atoms(ads)
        else:
            script[:0] = ['from ase.build import molecule']
            script += ['ads = molecule(%r)' % ads]
            ads = molecule(ads)

        add_adsorbate(slab, ads, 0.0, site)

        d = opt.distance
        if d is None:
            d = r + covalent_radii[ads[0].number] / 2
        
        h = opt.height
        if h is None:
            R = slab.positions
            y = ((R[:-nads] - R[-nads])**2).sum(1).min()**0.5
            h = (d**2 - y**2)**0.5
        else:
            assert opt.distance is None
        
        slab.positions[-nads:, 2] += h

        script[1] += ', add_adsorbate'
        script += ['add_adsorbate(slab, ads, %r, %r)' % (h, site)]
        
    if len(args) == 2:
        write(args[1], slab)
        script[1:1] = ['from ase.io import write']
        script += ['write(%r, slab)' % args[1]]
    elif not opt.gui:
        write(name + '.traj', slab)
        script[1:1] = ['from ase.io import write']
        script += ['write(%r, slab)' % (name + '.traj')]
        
    if opt.gui:
        view(slab)
        script[1:1] = ['from ase.visualize import view']
        script += ['view(slab)']

    if opt.python:
        print('\n'.join(script))


def estimate_lattice_constant(name, crystalstructure, covera):
    from ase.build import bulk
    atoms = bulk(name, crystalstructure, 1.0, covera)
    v0 = atoms.get_volume()
    v = 0.0
    for Z in atoms.get_atomic_numbers():
        r = covalent_radii[Z]
        v += 4 * np.pi / 3 * r**3 * 1.5
    return (v / v0)**(1.0 / 3)

    
if __name__ == '__main__':
    build()
